from uuid import uuid4

from scute import pack, _function_namespaces
from scute.internal.dictToNBT import dictToNBT
from scute.datasources import DataSource
from os.path import join
from inspect import signature
from typing import TypeVar
import os

from scute.internal.utils import formatText

_NbtSource = TypeVar('_NbtSource', bound=DataSource)

class _MacroArguments:
    """
    The class instance passed to a function that has been declared with arguments (a macro). Accessing any property will return a `_MacroArg` instance which can be passed into any command
    """
    def __getattr__(self, item):
        return _MacroArg(item)

class _MacroArg:
    def __init__(self, item):
        """
        An internal class returned from `_MacroArguments` which can be passed into any command
        """
        self.arg = f"$({item})"

    def __str__(self):
        return self.arg

def func(function_namespace=None, function_name=None):
    """
    A decorator that creates a function in your datapack. You can provide an optional namespace and name,
    or it will default to your main namespace and a randomised name.
    Args:
        function_namespace: The namespace your function should go under
        function_name: The name of your function

    .. include:: ../pdoc/documentation/functions.md
    """
    def decorator(function):
        if function not in _function_namespaces:
            sig = signature(function)
            if len(sig.parameters) > 0:
                function(_MacroArguments())
            else:
                function()
            name = function_name or uuid4()
            space = function_namespace or pack.namespace

            _function_namespaces[function] = f"{space}:{name}"

            bp = join(pack.path, pack.name)
            bp = join(bp, rf"data\{space}\functions")

            os.makedirs(bp, exist_ok=True)

            with open(join(bp, rf"{name}.mcfunction"), "w") as f:
                f.writelines(pack._command_stack)

            pack._command_stack = []

            print(formatText(f"Created function {space}:{name}", 32))

        # Code run if the function is called
        def wrapper(args: dict | _NbtSource = None, path=None):
            command = f"function {_function_namespaces[function]}"
            if isinstance(args, dict):
                command += " " + dictToNBT(args)
                if path is not None:
                    print("Warning: Macro argument path should not be specified when using hardcoded nbt.")
            elif issubclass(type(args), DataSource):
                command += f" with {args.str} {path}"
            pack._command_stack.append(command + "\n")

        wrapper.unwrapped = function

        return wrapper

    return decorator



import json
import os
from os.path import join

from scute import pack

def _createJsonFile(namespace, name, p, data):
    path = join(pack.path, pack.name)
    path = join(path, rf"data\{namespace}\{p}")

    os.makedirs(path, exist_ok=True)

    with open(join(path, name + ".json"), "w") as f:
        json.dump(data, f, indent=4)

def formatText(text, *codes):
    out = ""
    for code in codes:
        out += f"\033[{code}m"
    out += text
    return out + "\033[0m"



import json

from scute.datatypes import _NumberType


def encode_value(value):
    out = ""
    # If the value is a NBT number like 1b, get its NBT representation
    if issubclass(type(value), _NumberType):
        out += value.getNbt()
    # If the value is a dict or list, encode it
    elif isinstance(value, dict) or isinstance(value, list):
        out += encode_dict_or_list(value)
    # If the value is a string, just add it
    elif isinstance(value, str):
        escapedValue = value.replace("'", "\\'")
        out += f"'{escapedValue}'"
    # If the value is an int, just add it
    elif isinstance(value, int) or isinstance(value, float):
        out += str(value)

    return out


def encode_dict_or_list(obj):
    out = ""
    index = 0
    if isinstance(obj, dict):
        out = "{"
        # Loop through keys and values of the dict
        for key, value in obj.items():
            if index != 0:
                out += ", "

            # Add the key and value to the output
            out += str(key) + ": "
            out += encode_value(value)
            index += 1

        out += "}"

    elif isinstance(obj, list):
        out += "["
        for value in obj:
            if index != 0:
                out += ", "
            out += encode_value(value)

        out += "]"

    return out


class dtn(json.JSONEncoder):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def encode(self, obj):
        return encode_dict_or_list(obj)


def dictToNBT(dic):
    return json.dumps(dic, cls=dtn)



"""
Enum for all biomes in Minecraft.
"""
class Biome:
    badlands = "badlands"
    badlands_plateau = "badlands_plateau"
    bamboo_jungle = "bamboo_jungle"
    bamboo_jungle_hills = "bamboo_jungle_hills"
    basalt_deltas = "basalt_deltas"
    beach = "beach"
    birch_forest = "birch_forest"
    birch_forest_hills = "birch_forest_hills"
    cold_ocean = "cold_ocean"
    crimson_forest = "crimson_forest"
    dark_forest = "dark_forest"
    dark_forest_hills = "dark_forest_hills"
    deep_cold_ocean = "deep_cold_ocean"
    deep_frozen_ocean = "deep_frozen_ocean"
    deep_lukewarm_ocean = "deep_lukewarm_ocean"
    deep_ocean = "deep_ocean"
    deep_warm_ocean = "deep_warm_ocean"
    desert = "desert"
    desert_hills = "desert_hills"
    desert_lakes = "desert_lakes"
    end_barrens = "end_barrens"
    end_highlands = "end_highlands"
    end_midlands = "end_midlands"
    eroded_badlands = "eroded_badlands"
    flower_forest = "flower_forest"
    forest = "forest"
    frozen_ocean = "frozen_ocean"
    frozen_river = "frozen_river"
    giant_spruce_taiga = "giant_spruce_taiga"
    giant_spruce_taiga_hills = "giant_spruce_taiga_hills"
    giant_tree_taiga = "giant_tree_taiga"
    giant_tree_taiga_hills = "giant_tree_taiga_hills"
    gravelly_mountains = "gravelly_mountains"
    ice_spikes = "ice_spikes"
    jungle = "jungle"
    jungle_edge = "jungle_edge"
    jungle_hills = "jungle_hills"
    lukewarm_ocean = "lukewarm_ocean"
    modified_badlands_plateau = "modified_badlands_plateau"
    modified_gravelly_mountains = "modified_gravelly_mountains"
    modified_jungle = "modified_jungle"
    modified_jungle_edge = "modified_jungle_edge"
    modified_wooded_badlands_plateau = "modified_wooded_badlands_plateau"
    mountain_edge = "mountain_edge"
    mountains = "mountains"
    mushroom_field_shore = "mushroom_field_shore"
    mushroom_fields = "mushroom_fields"
    nether_wastes = "nether_wastes"
    ocean = "ocean"
    plains = "plains"
    river = "river"
    savanna = "savanna"
    savanna_plateau = "savanna_plateau"
    shattered_savanna = "shattered_savanna"
    shattered_savanna_plateau = "shattered_savanna_plateau"
    small_end_islands = "small_end_islands"
    snowy_beach = "snowy_beach"
    snowy_mountains = "snowy_mountains"
    snowy_taiga = "snowy_taiga"
    snowy_taiga_hills = "snowy_taiga_hills"
    snowy_taiga_mountains = "snowy_taiga_mountains"
    snowy_tundra = "snowy_tundra"
    soul_sand_valley = "soul_sand_valley"
    stone_shore = "stone_shore"
    sunflower_plains = "sunflower_plains"
    swamp = "swamp"
    swamp_hills = "swamp_hills"
    taiga = "taiga"
    taiga_hills = "taiga_hills"
    taiga_mountains = "taiga_mountains"
    tall_birch_forest = "tall_birch_forest"
    tall_birch_hills = "tall_birch_hills"
    the_end = "the_end"
    the_void = "the_void"
    warm_ocean = "warm_ocean"
    warped_forest = "warped_forest"
    wooded_badlands_plateau = "wooded_badlands_plateau"
    wooded_hills = "wooded_hills"
    wooded_mountains = "wooded_mountains"



"""
Submodule for data sources, like block entities, entities, and storage - eventually to be used in /data etc
"""
from scute import pack

class DataSource:
    pass

class Storage(DataSource):
    def __init__(self, namespace: str):
        """
        Returns a reference to an nbt storage for passing into /data etc
        Args:
            namespace: The namespace for the storage, like pack:name, or just a name under the default namespace
        """
        if ":" in namespace:
            self.namespace = namespace
        else:
            self.namespace = f"{pack.namespace}:{namespace}"
        self.str = f"storage {self.namespace}"

class BlockData(DataSource):
    def __init__(self, x, y, z):
        """
        Returns a reference to a block entity's data for passing into /data etc
        Args:
            x: The x coordinate of the block
            y: The y coordinate of the block
            z: The z coordinate of the block
        """
        self.x = x
        self.y = y
        self.z = z
        self.str = f"block {x} {y} {z}"

class EntityData(DataSource):
    def __init__(self, selector):
        """
        Returns a reference to an entity's data for passing into /data etc
        Args:
            selector: The selector of the entity
        """
        self.selector = selector
        self.str = f"entity {selector}"



"""
Enum for all enchantments in Minecraft.
"""
class Enchantment:
    aqua_affinity = "aqua_affinity"
    bane_of_arthropods = "bane_of_arthropods"
    binding_curse = "binding_curse"
    blast_protection = "blast_protection"
    channeling = "channeling"
    depth_strider = "depth_strider"
    efficiency = "efficiency"
    feather_falling = "feather_falling"
    fire_aspect = "fire_aspect"
    fire_protection = "fire_protection"
    flame = "flame"
    fortune = "fortune"
    frost_walker = "frost_walker"
    impaling = "impaling"
    infinity = "infinity"
    knockback = "knockback"
    looting = "looting"
    loyalty = "loyalty"
    luck_of_the_sea = "luck_of_the_sea"
    lure = "lure"
    mending = "mending"
    multishot = "multishot"
    piercing = "piercing"
    power = "power"
    projectile_protection = "projectile_protection"
    protection = "protection"
    punch = "punch"
    quick_charge = "quick_charge"
    respiration = "respiration"
    riptide = "riptide"
    sharpness = "sharpness"
    silk_touch = "silk_touch"
    smite = "smite"
    soul_speed = "soul_speed"
    sweeping = "sweeping"
    swift_sneak = "swift_sneak"
    thorns = "thorns"
    unbreaking = "unbreaking"
    vanishing_curse = "vanishing_curse"



"""
Submodule for creating and managing tags - function tags, block tags, item tags, etc.
"""
import atexit
import json
import os
from os.path import join

from scute import _function_namespaces, pack
from scute.internal.utils import _createJsonFile, formatText

_tags = {}

def _createFunctionTagFiles():
    for tag, functions in _tags.items():
        namespace, name = tag.split(":")
        _createJsonFile(namespace, name, r"tags\functions", {
                "values": [
                    func for func in functions
                ]
            })
        print(formatText(f"Successfully created function tag {namespace}:{name}", 32))

    print(formatText("Built!", 42, 30))


atexit.register(_createFunctionTagFiles)

def _addFuncToTag(func, tag, decoratorName):
    try:
        name = _function_namespaces[func.unwrapped]
        if tag in _tags:
            _tags[tag].append(name)
        else:
            _tags[tag] = [name]
    except:
        raise Exception(f"Please put @{decoratorName} decorators above @func")

def load(func):
    """
    Adds your function to load.json when used as a decorator
    """
    _addFuncToTag(func, "minecraft:load", "load")
    return func

def tick(func):
    """
    Adds your function to tick.json when used as a decorator
    """
    _addFuncToTag(func, "minecraft:tick", "tick")
    return func

def functionTag(namespace, name):
    """
    Adds your function to a function tag of your choosing when used as a decorator
    """
    def decorator(func):
        _addFuncToTag(func, f"{namespace}:{name}", "tag")
        return func
    return decorator

def createBlockTag(namespace: str, name: str, entries: list[str]):
    """
    Creates a block tag with the given entries
    Args:
        namespace: The namespace of the tag
        name: The name of the tag
        entries: The block ids of the tag
    """
    path = join(pack.path, pack.name)
    path = join(path, rf"data\{namespace}\tags\blocks")

    os.makedirs(path, exist_ok=True)

    with open(join(path, rf"{name}.json"), "w") as f:
        json.dump({
            "values": entries
        }, f, indent=4)

class BlockTag:
    def __init__(self, namespaceOrReference: str, name: str = None, entries: list[str] = None):
        """
        Creates or references a block tag. When called with one argument, it references an existing tag. When called with all of them, it creates one with the given entries.
        Args:
            namespaceOrReference: The namespace of the tag to be created or a reference to one, like "minecraft:wool"
            name: The name of the tag
            entries: The block ids of the tag
        """
        if entries is not None:
            path = join(pack.path, pack.name)
            path = join(path, rf"data\{namespaceOrReference}\tags\blocks")

            os.makedirs(path, exist_ok=True)

            with open(join(path, rf"{name}.json"), "w") as f:
                json.dump({
                    "values": entries
                }, f, indent=4)

            self.reference = f"{namespaceOrReference}:{name}"
            return

        self.reference = namespaceOrReference

class ItemTag:
    def __init__(self, namespaceOrReference: str, name: str = None, entries: list[str] = None):
        """
        Creates or references an item tag. When called with one argument, it references an existing tag. When called with all of them, it creates one with the given entries.
        Args:
            namespaceOrReference: The namespace of the tag to be created, like "mypack" or a reference to one, like "minecraft:wool"
            name: The name of the tag
            entries: The block ids of the tag
        """
        if entries:
            _createJsonFile(namespaceOrReference, name, r"tags\items", {
                "values": entries
            })
            self.reference = f"{namespaceOrReference}:{name}"
            return

        self.reference = namespaceOrReference




"""
Scute is a Python package designed to make making Minecraft datapacks easier. It allows you to create
almost all elements of a datapack with pure code (eventually - at the moment it's just functions and recipes).
.. include:: ../pdoc/documentation/index.md
"""

import json
import os
import shutil
from os.path import join

_function_namespaces = {}

class pack:
    meta = {"pack": {"pack_format": 1, "description": "My first pack"}}
    name = ""
    path = ""
    namespace = "scute"
    _command_stack = []

    @staticmethod
    def build():
        """
        Checks the validity of your pack, and creates the file structure. Must be run at the top of the file after you define pack.name, etc
        """
        if pack.name != "":
            if pack.path != "":
                bp = join(os.path.expandvars(pack.path), pack.name)
                try:
                    shutil.rmtree(bp)
                except:
                    pass
                os.makedirs(bp, exist_ok=True)
                try:
                    with open(join(bp, "pack.mcmeta"), "w") as mcmeta:
                        json.dump(pack.meta, mcmeta, indent=4)
                except Exception as e:
                    print("Build path is not valid, or folder does not exist. Error:")
                    print(e)
                    return

                os.makedirs(os.path.join(bp, "data"), exist_ok=True)

            else:
                print("Please set a path to compile to with scute.pack.setBuildPath()")
                return
        else:
            print("Please set a pack name with scute.pack.setName()")
            return

    @staticmethod
    def setName(name):
        """
        Sets the display name of the pack
        Args:
            name: The name
        """
        pack.name = name

    @staticmethod
    def setMainNamespace(namespace):
        """
        Sets the namespace that will be used for automatically-generated or anonymous functions
        Args:
            namespace: The namespace
        """
        pack.namespace = namespace

    @staticmethod
    def setDescription(desc: str):
        """
        Sets the description of your pack
        Args:
            desc: The description
        """
        pack.meta["pack"]["description"] = desc

    @staticmethod
    def setVersion(version: str | int):
        """
        Sets the version that the pack supports
        Args:
            version: Can be a major release like "1.19.4" (goes back to 1.16) or a pack_format number like 11
        """
        if isinstance(version, str):
            pack.meta["pack"]["pack_format"] = _versions[version]
        else:
            pack.meta["pack"]["pack_format"] = version

    @staticmethod
    def setBuildPath(path: str):
        """
        Sets the folder which your datapack will be built into - for example, "%appdata%/.minecraft/saves/world/datapacks", or "./output"
        """
        pack.path = os.path.expandvars(path)


class _JsonText:
    def __init__(self, text: dict):
        self.text = text

    def __str__(self):
        return json.dumps(self.text)


_versions = {
    "1.16": 5,
    "1.16.1": 5,
    "1.16.2": 6,
    "1.16.3": 6,
    "1.16.4": 6,
    "1.16.5": 6,
    "1.17": 7,
    "1.17.1": 7,
    "1.18": 8,
    "1.18.1": 8,
    "1.18.2": 9,
    "1.19": 10,
    "1.19.1": 10,
    "1.19.2": 10,
    "1.19.3": 10,
    "1.19.4": 12,
    "1.20": 15,
    "1.20.1": 15,
}



"""
Enum for item types, and a class for creating items.
"""
from scute import _JsonText
from scute.internal.dictToNBT import dictToNBT

def nbt(**kwargs):
    """
    An optional pretty-print way to define nbt data - `nbt(mytag=Byte(1))` is the same as `{"mytag": Byte(1)}`
    .. include:: ../pdoc/documentation/nbt.md
    """
    return kwargs


class Item:
    def __init__(self, id: str, nbt: dict = None, count=1, name: _JsonText | str = None):
        """
        Creates an item instance. Nbt can be specified manually with nbt= or using a utility, like name=
        Args:
            id: The id of the item, like Item.diamond
            nbt: Optional nbt for the item, like nbt(mytag=Byte(1)) or {"CustomModelData": 3}
            count: Optional count
            name: Utility for setting the name of the item
        """
        if ":" in id:
            self.id = id
        else:
            self.id = "minecraft:" + id
        if name is not None:
            if nbt is None:
                nbt = {}
            if "display" not in nbt:
                nbt["display"] = {}
            nbt["display"]["Name"] = str(name)
        self.nbt = nbt
        self.commandFormat = self.id + dictToNBT(nbt)
        self.count = count

    acacia_boat = "acacia_boat"
    acacia_button = "acacia_button"
    acacia_chest_boat = "acacia_chest_boat"
    acacia_door = "acacia_door"
    acacia_fence = "acacia_fence"
    acacia_fence_gate = "acacia_fence_gate"
    acacia_hanging_sign = "acacia_hanging_sign"
    acacia_leaves = "acacia_leaves"
    acacia_log = "acacia_log"
    acacia_planks = "acacia_planks"
    acacia_pressure_plate = "acacia_pressure_plate"
    acacia_sapling = "acacia_sapling"
    acacia_sign = "acacia_sign"
    acacia_slab = "acacia_slab"
    acacia_stairs = "acacia_stairs"
    acacia_trapdoor = "acacia_trapdoor"
    acacia_wood = "acacia_wood"
    activator_rail = "activator_rail"
    air = "air"
    allay_spawn_egg = "allay_spawn_egg"
    allium = "allium"
    amethyst_block = "amethyst_block"
    amethyst_cluster = "amethyst_cluster"
    amethyst_shard = "amethyst_shard"
    ancient_debris = "ancient_debris"
    andesite = "andesite"
    andesite_slab = "andesite_slab"
    andesite_stairs = "andesite_stairs"
    andesite_wall = "andesite_wall"
    angler_pottery_sherd = "angler_pottery_sherd"
    anvil = "anvil"
    apple = "apple"
    archer_pottery_sherd = "archer_pottery_sherd"
    armor_stand = "armor_stand"
    arms_up_pottery_sherd = "arms_up_pottery_sherd"
    arrow = "arrow"
    axolotl_bucket = "axolotl_bucket"
    axolotl_spawn_egg = "axolotl_spawn_egg"
    azalea = "azalea"
    azalea_leaves = "azalea_leaves"
    azure_bluet = "azure_bluet"
    baked_potato = "baked_potato"
    bamboo = "bamboo"
    bamboo_block = "bamboo_block"
    bamboo_button = "bamboo_button"
    bamboo_chest_raft = "bamboo_chest_raft"
    bamboo_door = "bamboo_door"
    bamboo_fence = "bamboo_fence"
    bamboo_fence_gate = "bamboo_fence_gate"
    bamboo_hanging_sign = "bamboo_hanging_sign"
    bamboo_mosaic = "bamboo_mosaic"
    bamboo_mosaic_slab = "bamboo_mosaic_slab"
    bamboo_mosaic_stairs = "bamboo_mosaic_stairs"
    bamboo_planks = "bamboo_planks"
    bamboo_pressure_plate = "bamboo_pressure_plate"
    bamboo_raft = "bamboo_raft"
    bamboo_sign = "bamboo_sign"
    bamboo_slab = "bamboo_slab"
    bamboo_stairs = "bamboo_stairs"
    bamboo_trapdoor = "bamboo_trapdoor"
    barrel = "barrel"
    barrier = "barrier"
    basalt = "basalt"
    bat_spawn_egg = "bat_spawn_egg"
    beacon = "beacon"
    bedrock = "bedrock"
    bee_nest = "bee_nest"
    bee_spawn_egg = "bee_spawn_egg"
    beef = "beef"
    beehive = "beehive"
    beetroot = "beetroot"
    beetroot_seeds = "beetroot_seeds"
    beetroot_soup = "beetroot_soup"
    bell = "bell"
    big_dripleaf = "big_dripleaf"
    birch_boat = "birch_boat"
    birch_button = "birch_button"
    birch_chest_boat = "birch_chest_boat"
    birch_door = "birch_door"
    birch_fence = "birch_fence"
    birch_fence_gate = "birch_fence_gate"
    birch_hanging_sign = "birch_hanging_sign"
    birch_leaves = "birch_leaves"
    birch_log = "birch_log"
    birch_planks = "birch_planks"
    birch_pressure_plate = "birch_pressure_plate"
    birch_sapling = "birch_sapling"
    birch_sign = "birch_sign"
    birch_slab = "birch_slab"
    birch_stairs = "birch_stairs"
    birch_trapdoor = "birch_trapdoor"
    birch_wood = "birch_wood"
    black_banner = "black_banner"
    black_bed = "black_bed"
    black_candle = "black_candle"
    black_carpet = "black_carpet"
    black_concrete = "black_concrete"
    black_concrete_powder = "black_concrete_powder"
    black_dye = "black_dye"
    black_glazed_terracotta = "black_glazed_terracotta"
    black_shulker_box = "black_shulker_box"
    black_stained_glass = "black_stained_glass"
    black_stained_glass_pane = "black_stained_glass_pane"
    black_terracotta = "black_terracotta"
    black_wool = "black_wool"
    blackstone = "blackstone"
    blackstone_slab = "blackstone_slab"
    blackstone_stairs = "blackstone_stairs"
    blackstone_wall = "blackstone_wall"
    blade_pottery_sherd = "blade_pottery_sherd"
    blast_furnace = "blast_furnace"
    blaze_powder = "blaze_powder"
    blaze_rod = "blaze_rod"
    blaze_spawn_egg = "blaze_spawn_egg"
    blue_banner = "blue_banner"
    blue_bed = "blue_bed"
    blue_candle = "blue_candle"
    blue_carpet = "blue_carpet"
    blue_concrete = "blue_concrete"
    blue_concrete_powder = "blue_concrete_powder"
    blue_dye = "blue_dye"
    blue_glazed_terracotta = "blue_glazed_terracotta"
    blue_ice = "blue_ice"
    blue_orchid = "blue_orchid"
    blue_shulker_box = "blue_shulker_box"
    blue_stained_glass = "blue_stained_glass"
    blue_stained_glass_pane = "blue_stained_glass_pane"
    blue_terracotta = "blue_terracotta"
    blue_wool = "blue_wool"
    bone = "bone"
    bone_block = "bone_block"
    bone_meal = "bone_meal"
    book = "book"
    bookshelf = "bookshelf"
    bow = "bow"
    bowl = "bowl"
    brain_coral = "brain_coral"
    brain_coral_block = "brain_coral_block"
    brain_coral_fan = "brain_coral_fan"
    bread = "bread"
    brewer_pottery_sherd = "brewer_pottery_sherd"
    brewing_stand = "brewing_stand"
    brick = "brick"
    brick_slab = "brick_slab"
    brick_stairs = "brick_stairs"
    brick_wall = "brick_wall"
    bricks = "bricks"
    brown_banner = "brown_banner"
    brown_bed = "brown_bed"
    brown_candle = "brown_candle"
    brown_carpet = "brown_carpet"
    brown_concrete = "brown_concrete"
    brown_concrete_powder = "brown_concrete_powder"
    brown_dye = "brown_dye"
    brown_glazed_terracotta = "brown_glazed_terracotta"
    brown_mushroom = "brown_mushroom"
    brown_mushroom_block = "brown_mushroom_block"
    brown_shulker_box = "brown_shulker_box"
    brown_stained_glass = "brown_stained_glass"
    brown_stained_glass_pane = "brown_stained_glass_pane"
    brown_terracotta = "brown_terracotta"
    brown_wool = "brown_wool"
    brush = "brush"
    bubble_coral = "bubble_coral"
    bubble_coral_block = "bubble_coral_block"
    bubble_coral_fan = "bubble_coral_fan"
    bucket = "bucket"
    budding_amethyst = "budding_amethyst"
    bundle = "bundle"
    burn_pottery_sherd = "burn_pottery_sherd"
    cactus = "cactus"
    cake = "cake"
    calcite = "calcite"
    calibrated_sculk_sensor = "calibrated_sculk_sensor"
    camel_spawn_egg = "camel_spawn_egg"
    campfire = "campfire"
    candle = "candle"
    carrot = "carrot"
    carrot_on_a_stick = "carrot_on_a_stick"
    cartography_table = "cartography_table"
    carved_pumpkin = "carved_pumpkin"
    cat_spawn_egg = "cat_spawn_egg"
    cauldron = "cauldron"
    cave_spider_spawn_egg = "cave_spider_spawn_egg"
    chain = "chain"
    chain_command_block = "chain_command_block"
    chainmail_boots = "chainmail_boots"
    chainmail_chestplate = "chainmail_chestplate"
    chainmail_helmet = "chainmail_helmet"
    chainmail_leggings = "chainmail_leggings"
    charcoal = "charcoal"
    cherry_boat = "cherry_boat"
    cherry_button = "cherry_button"
    cherry_chest_boat = "cherry_chest_boat"
    cherry_door = "cherry_door"
    cherry_fence = "cherry_fence"
    cherry_fence_gate = "cherry_fence_gate"
    cherry_hanging_sign = "cherry_hanging_sign"
    cherry_leaves = "cherry_leaves"
    cherry_log = "cherry_log"
    cherry_planks = "cherry_planks"
    cherry_pressure_plate = "cherry_pressure_plate"
    cherry_sapling = "cherry_sapling"
    cherry_sign = "cherry_sign"
    cherry_slab = "cherry_slab"
    cherry_stairs = "cherry_stairs"
    cherry_trapdoor = "cherry_trapdoor"
    cherry_wood = "cherry_wood"
    chest = "chest"
    chest_minecart = "chest_minecart"
    chicken = "chicken"
    chicken_spawn_egg = "chicken_spawn_egg"
    chipped_anvil = "chipped_anvil"
    chiseled_bookshelf = "chiseled_bookshelf"
    chiseled_deepslate = "chiseled_deepslate"
    chiseled_nether_bricks = "chiseled_nether_bricks"
    chiseled_polished_blackstone = "chiseled_polished_blackstone"
    chiseled_quartz_block = "chiseled_quartz_block"
    chiseled_red_sandstone = "chiseled_red_sandstone"
    chiseled_sandstone = "chiseled_sandstone"
    chiseled_stone_bricks = "chiseled_stone_bricks"
    chorus_flower = "chorus_flower"
    chorus_fruit = "chorus_fruit"
    chorus_plant = "chorus_plant"
    clay = "clay"
    clay_ball = "clay_ball"
    clock = "clock"
    coal = "coal"
    coal_block = "coal_block"
    coal_ore = "coal_ore"
    coarse_dirt = "coarse_dirt"
    coast_armor_trim_smithing_template = "coast_armor_trim_smithing_template"
    cobbled_deepslate = "cobbled_deepslate"
    cobbled_deepslate_slab = "cobbled_deepslate_slab"
    cobbled_deepslate_stairs = "cobbled_deepslate_stairs"
    cobbled_deepslate_wall = "cobbled_deepslate_wall"
    cobblestone = "cobblestone"
    cobblestone_slab = "cobblestone_slab"
    cobblestone_stairs = "cobblestone_stairs"
    cobblestone_wall = "cobblestone_wall"
    cobweb = "cobweb"
    cocoa_beans = "cocoa_beans"
    cod = "cod"
    cod_bucket = "cod_bucket"
    cod_spawn_egg = "cod_spawn_egg"
    command_block = "command_block"
    command_block_minecart = "command_block_minecart"
    comparator = "comparator"
    compass = "compass"
    composter = "composter"
    conduit = "conduit"
    cooked_beef = "cooked_beef"
    cooked_chicken = "cooked_chicken"
    cooked_cod = "cooked_cod"
    cooked_mutton = "cooked_mutton"
    cooked_porkchop = "cooked_porkchop"
    cooked_rabbit = "cooked_rabbit"
    cooked_salmon = "cooked_salmon"
    cookie = "cookie"
    copper_block = "copper_block"
    copper_ingot = "copper_ingot"
    copper_ore = "copper_ore"
    cornflower = "cornflower"
    cow_spawn_egg = "cow_spawn_egg"
    cracked_deepslate_bricks = "cracked_deepslate_bricks"
    cracked_deepslate_tiles = "cracked_deepslate_tiles"
    cracked_nether_bricks = "cracked_nether_bricks"
    cracked_polished_blackstone_bricks = "cracked_polished_blackstone_bricks"
    cracked_stone_bricks = "cracked_stone_bricks"
    crafting_table = "crafting_table"
    creeper_banner_pattern = "creeper_banner_pattern"
    creeper_head = "creeper_head"
    creeper_spawn_egg = "creeper_spawn_egg"
    crimson_button = "crimson_button"
    crimson_door = "crimson_door"
    crimson_fence = "crimson_fence"
    crimson_fence_gate = "crimson_fence_gate"
    crimson_fungus = "crimson_fungus"
    crimson_hanging_sign = "crimson_hanging_sign"
    crimson_hyphae = "crimson_hyphae"
    crimson_nylium = "crimson_nylium"
    crimson_planks = "crimson_planks"
    crimson_pressure_plate = "crimson_pressure_plate"
    crimson_roots = "crimson_roots"
    crimson_sign = "crimson_sign"
    crimson_slab = "crimson_slab"
    crimson_stairs = "crimson_stairs"
    crimson_stem = "crimson_stem"
    crimson_trapdoor = "crimson_trapdoor"
    crossbow = "crossbow"
    crying_obsidian = "crying_obsidian"
    cut_copper = "cut_copper"
    cut_copper_slab = "cut_copper_slab"
    cut_copper_stairs = "cut_copper_stairs"
    cut_red_sandstone = "cut_red_sandstone"
    cut_red_sandstone_slab = "cut_red_sandstone_slab"
    cut_sandstone = "cut_sandstone"
    cut_sandstone_slab = "cut_sandstone_slab"
    cyan_banner = "cyan_banner"
    cyan_bed = "cyan_bed"
    cyan_candle = "cyan_candle"
    cyan_carpet = "cyan_carpet"
    cyan_concrete = "cyan_concrete"
    cyan_concrete_powder = "cyan_concrete_powder"
    cyan_dye = "cyan_dye"
    cyan_glazed_terracotta = "cyan_glazed_terracotta"
    cyan_shulker_box = "cyan_shulker_box"
    cyan_stained_glass = "cyan_stained_glass"
    cyan_stained_glass_pane = "cyan_stained_glass_pane"
    cyan_terracotta = "cyan_terracotta"
    cyan_wool = "cyan_wool"
    damaged_anvil = "damaged_anvil"
    dandelion = "dandelion"
    danger_pottery_sherd = "danger_pottery_sherd"
    dark_oak_boat = "dark_oak_boat"
    dark_oak_button = "dark_oak_button"
    dark_oak_chest_boat = "dark_oak_chest_boat"
    dark_oak_door = "dark_oak_door"
    dark_oak_fence = "dark_oak_fence"
    dark_oak_fence_gate = "dark_oak_fence_gate"
    dark_oak_hanging_sign = "dark_oak_hanging_sign"
    dark_oak_leaves = "dark_oak_leaves"
    dark_oak_log = "dark_oak_log"
    dark_oak_planks = "dark_oak_planks"
    dark_oak_pressure_plate = "dark_oak_pressure_plate"
    dark_oak_sapling = "dark_oak_sapling"
    dark_oak_sign = "dark_oak_sign"
    dark_oak_slab = "dark_oak_slab"
    dark_oak_stairs = "dark_oak_stairs"
    dark_oak_trapdoor = "dark_oak_trapdoor"
    dark_oak_wood = "dark_oak_wood"
    dark_prismarine = "dark_prismarine"
    dark_prismarine_slab = "dark_prismarine_slab"
    dark_prismarine_stairs = "dark_prismarine_stairs"
    daylight_detector = "daylight_detector"
    dead_brain_coral = "dead_brain_coral"
    dead_brain_coral_block = "dead_brain_coral_block"
    dead_brain_coral_fan = "dead_brain_coral_fan"
    dead_bubble_coral = "dead_bubble_coral"
    dead_bubble_coral_block = "dead_bubble_coral_block"
    dead_bubble_coral_fan = "dead_bubble_coral_fan"
    dead_bush = "dead_bush"
    dead_fire_coral = "dead_fire_coral"
    dead_fire_coral_block = "dead_fire_coral_block"
    dead_fire_coral_fan = "dead_fire_coral_fan"
    dead_horn_coral = "dead_horn_coral"
    dead_horn_coral_block = "dead_horn_coral_block"
    dead_horn_coral_fan = "dead_horn_coral_fan"
    dead_tube_coral = "dead_tube_coral"
    dead_tube_coral_block = "dead_tube_coral_block"
    dead_tube_coral_fan = "dead_tube_coral_fan"
    debug_stick = "debug_stick"
    decorated_pot = "decorated_pot"
    deepslate = "deepslate"
    deepslate_brick_slab = "deepslate_brick_slab"
    deepslate_brick_stairs = "deepslate_brick_stairs"
    deepslate_brick_wall = "deepslate_brick_wall"
    deepslate_bricks = "deepslate_bricks"
    deepslate_coal_ore = "deepslate_coal_ore"
    deepslate_copper_ore = "deepslate_copper_ore"
    deepslate_diamond_ore = "deepslate_diamond_ore"
    deepslate_emerald_ore = "deepslate_emerald_ore"
    deepslate_gold_ore = "deepslate_gold_ore"
    deepslate_iron_ore = "deepslate_iron_ore"
    deepslate_lapis_ore = "deepslate_lapis_ore"
    deepslate_redstone_ore = "deepslate_redstone_ore"
    deepslate_tile_slab = "deepslate_tile_slab"
    deepslate_tile_stairs = "deepslate_tile_stairs"
    deepslate_tile_wall = "deepslate_tile_wall"
    deepslate_tiles = "deepslate_tiles"
    detector_rail = "detector_rail"
    diamond = "diamond"
    diamond_axe = "diamond_axe"
    diamond_block = "diamond_block"
    diamond_boots = "diamond_boots"
    diamond_chestplate = "diamond_chestplate"
    diamond_helmet = "diamond_helmet"
    diamond_hoe = "diamond_hoe"
    diamond_horse_armor = "diamond_horse_armor"
    diamond_leggings = "diamond_leggings"
    diamond_ore = "diamond_ore"
    diamond_pickaxe = "diamond_pickaxe"
    diamond_shovel = "diamond_shovel"
    diamond_sword = "diamond_sword"
    diorite = "diorite"
    diorite_slab = "diorite_slab"
    diorite_stairs = "diorite_stairs"
    diorite_wall = "diorite_wall"
    dirt = "dirt"
    dirt_path = "dirt_path"
    disc_fragment_5 = "disc_fragment_5"
    dispenser = "dispenser"
    dolphin_spawn_egg = "dolphin_spawn_egg"
    donkey_spawn_egg = "donkey_spawn_egg"
    dragon_breath = "dragon_breath"
    dragon_egg = "dragon_egg"
    dragon_head = "dragon_head"
    dried_kelp = "dried_kelp"
    dried_kelp_block = "dried_kelp_block"
    dripstone_block = "dripstone_block"
    dropper = "dropper"
    drowned_spawn_egg = "drowned_spawn_egg"
    dune_armor_trim_smithing_template = "dune_armor_trim_smithing_template"
    echo_shard = "echo_shard"
    egg = "egg"
    elder_guardian_spawn_egg = "elder_guardian_spawn_egg"
    elytra = "elytra"
    emerald = "emerald"
    emerald_block = "emerald_block"
    emerald_ore = "emerald_ore"
    enchanted_book = "enchanted_book"
    enchanted_golden_apple = "enchanted_golden_apple"
    enchanting_table = "enchanting_table"
    end_crystal = "end_crystal"
    end_portal_frame = "end_portal_frame"
    end_rod = "end_rod"
    end_stone = "end_stone"
    end_stone_brick_slab = "end_stone_brick_slab"
    end_stone_brick_stairs = "end_stone_brick_stairs"
    end_stone_brick_wall = "end_stone_brick_wall"
    end_stone_bricks = "end_stone_bricks"
    ender_chest = "ender_chest"
    ender_dragon_spawn_egg = "ender_dragon_spawn_egg"
    ender_eye = "ender_eye"
    ender_pearl = "ender_pearl"
    enderman_spawn_egg = "enderman_spawn_egg"
    endermite_spawn_egg = "endermite_spawn_egg"
    evoker_spawn_egg = "evoker_spawn_egg"
    experience_bottle = "experience_bottle"
    explorer_pottery_sherd = "explorer_pottery_sherd"
    exposed_copper = "exposed_copper"
    exposed_cut_copper = "exposed_cut_copper"
    exposed_cut_copper_slab = "exposed_cut_copper_slab"
    exposed_cut_copper_stairs = "exposed_cut_copper_stairs"
    eye_armor_trim_smithing_template = "eye_armor_trim_smithing_template"
    farmland = "farmland"
    feather = "feather"
    fermented_spider_eye = "fermented_spider_eye"
    fern = "fern"
    filled_map = "filled_map"
    fire_charge = "fire_charge"
    fire_coral = "fire_coral"
    fire_coral_block = "fire_coral_block"
    fire_coral_fan = "fire_coral_fan"
    firework_rocket = "firework_rocket"
    firework_star = "firework_star"
    fishing_rod = "fishing_rod"
    fletching_table = "fletching_table"
    flint = "flint"
    flint_and_steel = "flint_and_steel"
    flower_banner_pattern = "flower_banner_pattern"
    flower_pot = "flower_pot"
    flowering_azalea = "flowering_azalea"
    flowering_azalea_leaves = "flowering_azalea_leaves"
    fox_spawn_egg = "fox_spawn_egg"
    friend_pottery_sherd = "friend_pottery_sherd"
    frog_spawn_egg = "frog_spawn_egg"
    frogspawn = "frogspawn"
    furnace = "furnace"
    furnace_minecart = "furnace_minecart"
    ghast_spawn_egg = "ghast_spawn_egg"
    ghast_tear = "ghast_tear"
    gilded_blackstone = "gilded_blackstone"
    glass = "glass"
    glass_bottle = "glass_bottle"
    glass_pane = "glass_pane"
    glistering_melon_slice = "glistering_melon_slice"
    globe_banner_pattern = "globe_banner_pattern"
    glow_berries = "glow_berries"
    glow_ink_sac = "glow_ink_sac"
    glow_item_frame = "glow_item_frame"
    glow_lichen = "glow_lichen"
    glow_squid_spawn_egg = "glow_squid_spawn_egg"
    glowstone = "glowstone"
    glowstone_dust = "glowstone_dust"
    goat_horn = "goat_horn"
    goat_spawn_egg = "goat_spawn_egg"
    gold_block = "gold_block"
    gold_ingot = "gold_ingot"
    gold_nugget = "gold_nugget"
    gold_ore = "gold_ore"
    golden_apple = "golden_apple"
    golden_axe = "golden_axe"
    golden_boots = "golden_boots"
    golden_carrot = "golden_carrot"
    golden_chestplate = "golden_chestplate"
    golden_helmet = "golden_helmet"
    golden_hoe = "golden_hoe"
    golden_horse_armor = "golden_horse_armor"
    golden_leggings = "golden_leggings"
    golden_pickaxe = "golden_pickaxe"
    golden_shovel = "golden_shovel"
    golden_sword = "golden_sword"
    granite = "granite"
    granite_slab = "granite_slab"
    granite_stairs = "granite_stairs"
    granite_wall = "granite_wall"
    grass = "grass"
    grass_block = "grass_block"
    gravel = "gravel"
    gray_banner = "gray_banner"
    gray_bed = "gray_bed"
    gray_candle = "gray_candle"
    gray_carpet = "gray_carpet"
    gray_concrete = "gray_concrete"
    gray_concrete_powder = "gray_concrete_powder"
    gray_dye = "gray_dye"
    gray_glazed_terracotta = "gray_glazed_terracotta"
    gray_shulker_box = "gray_shulker_box"
    gray_stained_glass = "gray_stained_glass"
    gray_stained_glass_pane = "gray_stained_glass_pane"
    gray_terracotta = "gray_terracotta"
    gray_wool = "gray_wool"
    green_banner = "green_banner"
    green_bed = "green_bed"
    green_candle = "green_candle"
    green_carpet = "green_carpet"
    green_concrete = "green_concrete"
    green_concrete_powder = "green_concrete_powder"
    green_dye = "green_dye"
    green_glazed_terracotta = "green_glazed_terracotta"
    green_shulker_box = "green_shulker_box"
    green_stained_glass = "green_stained_glass"
    green_stained_glass_pane = "green_stained_glass_pane"
    green_terracotta = "green_terracotta"
    green_wool = "green_wool"
    grindstone = "grindstone"
    guardian_spawn_egg = "guardian_spawn_egg"
    gunpowder = "gunpowder"
    hanging_roots = "hanging_roots"
    hay_block = "hay_block"
    heart_of_the_sea = "heart_of_the_sea"
    heart_pottery_sherd = "heart_pottery_sherd"
    heartbreak_pottery_sherd = "heartbreak_pottery_sherd"
    heavy_weighted_pressure_plate = "heavy_weighted_pressure_plate"
    hoglin_spawn_egg = "hoglin_spawn_egg"
    honey_block = "honey_block"
    honey_bottle = "honey_bottle"
    honeycomb = "honeycomb"
    honeycomb_block = "honeycomb_block"
    hopper = "hopper"
    hopper_minecart = "hopper_minecart"
    horn_coral = "horn_coral"
    horn_coral_block = "horn_coral_block"
    horn_coral_fan = "horn_coral_fan"
    horse_spawn_egg = "horse_spawn_egg"
    host_armor_trim_smithing_template = "host_armor_trim_smithing_template"
    howl_pottery_sherd = "howl_pottery_sherd"
    husk_spawn_egg = "husk_spawn_egg"
    ice = "ice"
    infested_chiseled_stone_bricks = "infested_chiseled_stone_bricks"
    infested_cobblestone = "infested_cobblestone"
    infested_cracked_stone_bricks = "infested_cracked_stone_bricks"
    infested_deepslate = "infested_deepslate"
    infested_mossy_stone_bricks = "infested_mossy_stone_bricks"
    infested_stone = "infested_stone"
    infested_stone_bricks = "infested_stone_bricks"
    ink_sac = "ink_sac"
    iron_axe = "iron_axe"
    iron_bars = "iron_bars"
    iron_block = "iron_block"
    iron_boots = "iron_boots"
    iron_chestplate = "iron_chestplate"
    iron_door = "iron_door"
    iron_golem_spawn_egg = "iron_golem_spawn_egg"
    iron_helmet = "iron_helmet"
    iron_hoe = "iron_hoe"
    iron_horse_armor = "iron_horse_armor"
    iron_ingot = "iron_ingot"
    iron_leggings = "iron_leggings"
    iron_nugget = "iron_nugget"
    iron_ore = "iron_ore"
    iron_pickaxe = "iron_pickaxe"
    iron_shovel = "iron_shovel"
    iron_sword = "iron_sword"
    iron_trapdoor = "iron_trapdoor"
    item_frame = "item_frame"
    jack_o_lantern = "jack_o_lantern"
    jigsaw = "jigsaw"
    jukebox = "jukebox"
    jungle_boat = "jungle_boat"
    jungle_button = "jungle_button"
    jungle_chest_boat = "jungle_chest_boat"
    jungle_door = "jungle_door"
    jungle_fence = "jungle_fence"
    jungle_fence_gate = "jungle_fence_gate"
    jungle_hanging_sign = "jungle_hanging_sign"
    jungle_leaves = "jungle_leaves"
    jungle_log = "jungle_log"
    jungle_planks = "jungle_planks"
    jungle_pressure_plate = "jungle_pressure_plate"
    jungle_sapling = "jungle_sapling"
    jungle_sign = "jungle_sign"
    jungle_slab = "jungle_slab"
    jungle_stairs = "jungle_stairs"
    jungle_trapdoor = "jungle_trapdoor"
    jungle_wood = "jungle_wood"
    kelp = "kelp"
    knowledge_book = "knowledge_book"
    ladder = "ladder"
    lantern = "lantern"
    lapis_block = "lapis_block"
    lapis_lazuli = "lapis_lazuli"
    lapis_ore = "lapis_ore"
    large_amethyst_bud = "large_amethyst_bud"
    large_fern = "large_fern"
    lava_bucket = "lava_bucket"
    lead = "lead"
    leather = "leather"
    leather_boots = "leather_boots"
    leather_chestplate = "leather_chestplate"
    leather_helmet = "leather_helmet"
    leather_horse_armor = "leather_horse_armor"
    leather_leggings = "leather_leggings"
    lectern = "lectern"
    lever = "lever"
    light = "light"
    light_blue_banner = "light_blue_banner"
    light_blue_bed = "light_blue_bed"
    light_blue_candle = "light_blue_candle"
    light_blue_carpet = "light_blue_carpet"
    light_blue_concrete = "light_blue_concrete"
    light_blue_concrete_powder = "light_blue_concrete_powder"
    light_blue_dye = "light_blue_dye"
    light_blue_glazed_terracotta = "light_blue_glazed_terracotta"
    light_blue_shulker_box = "light_blue_shulker_box"
    light_blue_stained_glass = "light_blue_stained_glass"
    light_blue_stained_glass_pane = "light_blue_stained_glass_pane"
    light_blue_terracotta = "light_blue_terracotta"
    light_blue_wool = "light_blue_wool"
    light_gray_banner = "light_gray_banner"
    light_gray_bed = "light_gray_bed"
    light_gray_candle = "light_gray_candle"
    light_gray_carpet = "light_gray_carpet"
    light_gray_concrete = "light_gray_concrete"
    light_gray_concrete_powder = "light_gray_concrete_powder"
    light_gray_dye = "light_gray_dye"
    light_gray_glazed_terracotta = "light_gray_glazed_terracotta"
    light_gray_shulker_box = "light_gray_shulker_box"
    light_gray_stained_glass = "light_gray_stained_glass"
    light_gray_stained_glass_pane = "light_gray_stained_glass_pane"
    light_gray_terracotta = "light_gray_terracotta"
    light_gray_wool = "light_gray_wool"
    light_weighted_pressure_plate = "light_weighted_pressure_plate"
    lightning_rod = "lightning_rod"
    lilac = "lilac"
    lily_of_the_valley = "lily_of_the_valley"
    lily_pad = "lily_pad"
    lime_banner = "lime_banner"
    lime_bed = "lime_bed"
    lime_candle = "lime_candle"
    lime_carpet = "lime_carpet"
    lime_concrete = "lime_concrete"
    lime_concrete_powder = "lime_concrete_powder"
    lime_dye = "lime_dye"
    lime_glazed_terracotta = "lime_glazed_terracotta"
    lime_shulker_box = "lime_shulker_box"
    lime_stained_glass = "lime_stained_glass"
    lime_stained_glass_pane = "lime_stained_glass_pane"
    lime_terracotta = "lime_terracotta"
    lime_wool = "lime_wool"
    lingering_potion = "lingering_potion"
    llama_spawn_egg = "llama_spawn_egg"
    lodestone = "lodestone"
    loom = "loom"
    magenta_banner = "magenta_banner"
    magenta_bed = "magenta_bed"
    magenta_candle = "magenta_candle"
    magenta_carpet = "magenta_carpet"
    magenta_concrete = "magenta_concrete"
    magenta_concrete_powder = "magenta_concrete_powder"
    magenta_dye = "magenta_dye"
    magenta_glazed_terracotta = "magenta_glazed_terracotta"
    magenta_shulker_box = "magenta_shulker_box"
    magenta_stained_glass = "magenta_stained_glass"
    magenta_stained_glass_pane = "magenta_stained_glass_pane"
    magenta_terracotta = "magenta_terracotta"
    magenta_wool = "magenta_wool"
    magma_block = "magma_block"
    magma_cream = "magma_cream"
    magma_cube_spawn_egg = "magma_cube_spawn_egg"
    mangrove_boat = "mangrove_boat"
    mangrove_button = "mangrove_button"
    mangrove_chest_boat = "mangrove_chest_boat"
    mangrove_door = "mangrove_door"
    mangrove_fence = "mangrove_fence"
    mangrove_fence_gate = "mangrove_fence_gate"
    mangrove_hanging_sign = "mangrove_hanging_sign"
    mangrove_leaves = "mangrove_leaves"
    mangrove_log = "mangrove_log"
    mangrove_planks = "mangrove_planks"
    mangrove_pressure_plate = "mangrove_pressure_plate"
    mangrove_propagule = "mangrove_propagule"
    mangrove_roots = "mangrove_roots"
    mangrove_sign = "mangrove_sign"
    mangrove_slab = "mangrove_slab"
    mangrove_stairs = "mangrove_stairs"
    mangrove_trapdoor = "mangrove_trapdoor"
    mangrove_wood = "mangrove_wood"
    map = "map"
    medium_amethyst_bud = "medium_amethyst_bud"
    melon = "melon"
    melon_seeds = "melon_seeds"
    melon_slice = "melon_slice"
    milk_bucket = "milk_bucket"
    minecart = "minecart"
    miner_pottery_sherd = "miner_pottery_sherd"
    mojang_banner_pattern = "mojang_banner_pattern"
    mooshroom_spawn_egg = "mooshroom_spawn_egg"
    moss_block = "moss_block"
    moss_carpet = "moss_carpet"
    mossy_cobblestone = "mossy_cobblestone"
    mossy_cobblestone_slab = "mossy_cobblestone_slab"
    mossy_cobblestone_stairs = "mossy_cobblestone_stairs"
    mossy_cobblestone_wall = "mossy_cobblestone_wall"
    mossy_stone_brick_slab = "mossy_stone_brick_slab"
    mossy_stone_brick_stairs = "mossy_stone_brick_stairs"
    mossy_stone_brick_wall = "mossy_stone_brick_wall"
    mossy_stone_bricks = "mossy_stone_bricks"
    mourner_pottery_sherd = "mourner_pottery_sherd"
    mud = "mud"
    mud_brick_slab = "mud_brick_slab"
    mud_brick_stairs = "mud_brick_stairs"
    mud_brick_wall = "mud_brick_wall"
    mud_bricks = "mud_bricks"
    muddy_mangrove_roots = "muddy_mangrove_roots"
    mule_spawn_egg = "mule_spawn_egg"
    mushroom_stem = "mushroom_stem"
    mushroom_stew = "mushroom_stew"
    music_disc_11 = "music_disc_11"
    music_disc_13 = "music_disc_13"
    music_disc_5 = "music_disc_5"
    music_disc_blocks = "music_disc_blocks"
    music_disc_cat = "music_disc_cat"
    music_disc_chirp = "music_disc_chirp"
    music_disc_far = "music_disc_far"
    music_disc_mall = "music_disc_mall"
    music_disc_mellohi = "music_disc_mellohi"
    music_disc_otherside = "music_disc_otherside"
    music_disc_pigstep = "music_disc_pigstep"
    music_disc_relic = "music_disc_relic"
    music_disc_stal = "music_disc_stal"
    music_disc_strad = "music_disc_strad"
    music_disc_wait = "music_disc_wait"
    music_disc_ward = "music_disc_ward"
    mutton = "mutton"
    mycelium = "mycelium"
    name_tag = "name_tag"
    nautilus_shell = "nautilus_shell"
    nether_brick = "nether_brick"
    nether_brick_fence = "nether_brick_fence"
    nether_brick_slab = "nether_brick_slab"
    nether_brick_stairs = "nether_brick_stairs"
    nether_brick_wall = "nether_brick_wall"
    nether_bricks = "nether_bricks"
    nether_gold_ore = "nether_gold_ore"
    nether_quartz_ore = "nether_quartz_ore"
    nether_sprouts = "nether_sprouts"
    nether_star = "nether_star"
    nether_wart = "nether_wart"
    nether_wart_block = "nether_wart_block"
    netherite_axe = "netherite_axe"
    netherite_block = "netherite_block"
    netherite_boots = "netherite_boots"
    netherite_chestplate = "netherite_chestplate"
    netherite_helmet = "netherite_helmet"
    netherite_hoe = "netherite_hoe"
    netherite_ingot = "netherite_ingot"
    netherite_leggings = "netherite_leggings"
    netherite_pickaxe = "netherite_pickaxe"
    netherite_scrap = "netherite_scrap"
    netherite_shovel = "netherite_shovel"
    netherite_sword = "netherite_sword"
    netherite_upgrade_smithing_template = "netherite_upgrade_smithing_template"
    netherrack = "netherrack"
    note_block = "note_block"
    oak_boat = "oak_boat"
    oak_button = "oak_button"
    oak_chest_boat = "oak_chest_boat"
    oak_door = "oak_door"
    oak_fence = "oak_fence"
    oak_fence_gate = "oak_fence_gate"
    oak_hanging_sign = "oak_hanging_sign"
    oak_leaves = "oak_leaves"
    oak_log = "oak_log"
    oak_planks = "oak_planks"
    oak_pressure_plate = "oak_pressure_plate"
    oak_sapling = "oak_sapling"
    oak_sign = "oak_sign"
    oak_slab = "oak_slab"
    oak_stairs = "oak_stairs"
    oak_trapdoor = "oak_trapdoor"
    oak_wood = "oak_wood"
    observer = "observer"
    obsidian = "obsidian"
    ocelot_spawn_egg = "ocelot_spawn_egg"
    ochre_froglight = "ochre_froglight"
    orange_banner = "orange_banner"
    orange_bed = "orange_bed"
    orange_candle = "orange_candle"
    orange_carpet = "orange_carpet"
    orange_concrete = "orange_concrete"
    orange_concrete_powder = "orange_concrete_powder"
    orange_dye = "orange_dye"
    orange_glazed_terracotta = "orange_glazed_terracotta"
    orange_shulker_box = "orange_shulker_box"
    orange_stained_glass = "orange_stained_glass"
    orange_stained_glass_pane = "orange_stained_glass_pane"
    orange_terracotta = "orange_terracotta"
    orange_tulip = "orange_tulip"
    orange_wool = "orange_wool"
    oxeye_daisy = "oxeye_daisy"
    oxidized_copper = "oxidized_copper"
    oxidized_cut_copper = "oxidized_cut_copper"
    oxidized_cut_copper_slab = "oxidized_cut_copper_slab"
    oxidized_cut_copper_stairs = "oxidized_cut_copper_stairs"
    packed_ice = "packed_ice"
    packed_mud = "packed_mud"
    painting = "painting"
    panda_spawn_egg = "panda_spawn_egg"
    paper = "paper"
    parrot_spawn_egg = "parrot_spawn_egg"
    pearlescent_froglight = "pearlescent_froglight"
    peony = "peony"
    petrified_oak_slab = "petrified_oak_slab"
    phantom_membrane = "phantom_membrane"
    phantom_spawn_egg = "phantom_spawn_egg"
    pig_spawn_egg = "pig_spawn_egg"
    piglin_banner_pattern = "piglin_banner_pattern"
    piglin_brute_spawn_egg = "piglin_brute_spawn_egg"
    piglin_head = "piglin_head"
    piglin_spawn_egg = "piglin_spawn_egg"
    pillager_spawn_egg = "pillager_spawn_egg"
    pink_banner = "pink_banner"
    pink_bed = "pink_bed"
    pink_candle = "pink_candle"
    pink_carpet = "pink_carpet"
    pink_concrete = "pink_concrete"
    pink_concrete_powder = "pink_concrete_powder"
    pink_dye = "pink_dye"
    pink_glazed_terracotta = "pink_glazed_terracotta"
    pink_petals = "pink_petals"
    pink_shulker_box = "pink_shulker_box"
    pink_stained_glass = "pink_stained_glass"
    pink_stained_glass_pane = "pink_stained_glass_pane"
    pink_terracotta = "pink_terracotta"
    pink_tulip = "pink_tulip"
    pink_wool = "pink_wool"
    piston = "piston"
    pitcher_plant = "pitcher_plant"
    pitcher_pod = "pitcher_pod"
    player_head = "player_head"
    plenty_pottery_sherd = "plenty_pottery_sherd"
    podzol = "podzol"
    pointed_dripstone = "pointed_dripstone"
    poisonous_potato = "poisonous_potato"
    polar_bear_spawn_egg = "polar_bear_spawn_egg"
    polished_andesite = "polished_andesite"
    polished_andesite_slab = "polished_andesite_slab"
    polished_andesite_stairs = "polished_andesite_stairs"
    polished_basalt = "polished_basalt"
    polished_blackstone = "polished_blackstone"
    polished_blackstone_brick_slab = "polished_blackstone_brick_slab"
    polished_blackstone_brick_stairs = "polished_blackstone_brick_stairs"
    polished_blackstone_brick_wall = "polished_blackstone_brick_wall"
    polished_blackstone_bricks = "polished_blackstone_bricks"
    polished_blackstone_button = "polished_blackstone_button"
    polished_blackstone_pressure_plate = "polished_blackstone_pressure_plate"
    polished_blackstone_slab = "polished_blackstone_slab"
    polished_blackstone_stairs = "polished_blackstone_stairs"
    polished_blackstone_wall = "polished_blackstone_wall"
    polished_deepslate = "polished_deepslate"
    polished_deepslate_slab = "polished_deepslate_slab"
    polished_deepslate_stairs = "polished_deepslate_stairs"
    polished_deepslate_wall = "polished_deepslate_wall"
    polished_diorite = "polished_diorite"
    polished_diorite_slab = "polished_diorite_slab"
    polished_diorite_stairs = "polished_diorite_stairs"
    polished_granite = "polished_granite"
    polished_granite_slab = "polished_granite_slab"
    polished_granite_stairs = "polished_granite_stairs"
    popped_chorus_fruit = "popped_chorus_fruit"
    poppy = "poppy"
    porkchop = "porkchop"
    potato = "potato"
    potion = "potion"
    powder_snow_bucket = "powder_snow_bucket"
    powered_rail = "powered_rail"
    prismarine = "prismarine"
    prismarine_brick_slab = "prismarine_brick_slab"
    prismarine_brick_stairs = "prismarine_brick_stairs"
    prismarine_bricks = "prismarine_bricks"
    prismarine_crystals = "prismarine_crystals"
    prismarine_shard = "prismarine_shard"
    prismarine_slab = "prismarine_slab"
    prismarine_stairs = "prismarine_stairs"
    prismarine_wall = "prismarine_wall"
    prize_pottery_sherd = "prize_pottery_sherd"
    pufferfish = "pufferfish"
    pufferfish_bucket = "pufferfish_bucket"
    pufferfish_spawn_egg = "pufferfish_spawn_egg"
    pumpkin = "pumpkin"
    pumpkin_pie = "pumpkin_pie"
    pumpkin_seeds = "pumpkin_seeds"
    purple_banner = "purple_banner"
    purple_bed = "purple_bed"
    purple_candle = "purple_candle"
    purple_carpet = "purple_carpet"
    purple_concrete = "purple_concrete"
    purple_concrete_powder = "purple_concrete_powder"
    purple_dye = "purple_dye"
    purple_glazed_terracotta = "purple_glazed_terracotta"
    purple_shulker_box = "purple_shulker_box"
    purple_stained_glass = "purple_stained_glass"
    purple_stained_glass_pane = "purple_stained_glass_pane"
    purple_terracotta = "purple_terracotta"
    purple_wool = "purple_wool"
    purpur_block = "purpur_block"
    purpur_pillar = "purpur_pillar"
    purpur_slab = "purpur_slab"
    purpur_stairs = "purpur_stairs"
    quartz = "quartz"
    quartz_block = "quartz_block"
    quartz_bricks = "quartz_bricks"
    quartz_pillar = "quartz_pillar"
    quartz_slab = "quartz_slab"
    quartz_stairs = "quartz_stairs"
    rabbit = "rabbit"
    rabbit_foot = "rabbit_foot"
    rabbit_hide = "rabbit_hide"
    rabbit_spawn_egg = "rabbit_spawn_egg"
    rabbit_stew = "rabbit_stew"
    rail = "rail"
    raiser_armor_trim_smithing_template = "raiser_armor_trim_smithing_template"
    ravager_spawn_egg = "ravager_spawn_egg"
    raw_copper = "raw_copper"
    raw_copper_block = "raw_copper_block"
    raw_gold = "raw_gold"
    raw_gold_block = "raw_gold_block"
    raw_iron = "raw_iron"
    raw_iron_block = "raw_iron_block"
    recovery_compass = "recovery_compass"
    red_banner = "red_banner"
    red_bed = "red_bed"
    red_candle = "red_candle"
    red_carpet = "red_carpet"
    red_concrete = "red_concrete"
    red_concrete_powder = "red_concrete_powder"
    red_dye = "red_dye"
    red_glazed_terracotta = "red_glazed_terracotta"
    red_mushroom = "red_mushroom"
    red_mushroom_block = "red_mushroom_block"
    red_nether_brick_slab = "red_nether_brick_slab"
    red_nether_brick_stairs = "red_nether_brick_stairs"
    red_nether_brick_wall = "red_nether_brick_wall"
    red_nether_bricks = "red_nether_bricks"
    red_sand = "red_sand"
    red_sandstone = "red_sandstone"
    red_sandstone_slab = "red_sandstone_slab"
    red_sandstone_stairs = "red_sandstone_stairs"
    red_sandstone_wall = "red_sandstone_wall"
    red_shulker_box = "red_shulker_box"
    red_stained_glass = "red_stained_glass"
    red_stained_glass_pane = "red_stained_glass_pane"
    red_terracotta = "red_terracotta"
    red_tulip = "red_tulip"
    red_wool = "red_wool"
    redstone = "redstone"
    redstone_block = "redstone_block"
    redstone_lamp = "redstone_lamp"
    redstone_ore = "redstone_ore"
    redstone_torch = "redstone_torch"
    reinforced_deepslate = "reinforced_deepslate"
    repeater = "repeater"
    repeating_command_block = "repeating_command_block"
    respawn_anchor = "respawn_anchor"
    rib_armor_trim_smithing_template = "rib_armor_trim_smithing_template"
    rooted_dirt = "rooted_dirt"
    rose_bush = "rose_bush"
    rotten_flesh = "rotten_flesh"
    saddle = "saddle"
    salmon = "salmon"
    salmon_bucket = "salmon_bucket"
    salmon_spawn_egg = "salmon_spawn_egg"
    sand = "sand"
    sandstone = "sandstone"
    sandstone_slab = "sandstone_slab"
    sandstone_stairs = "sandstone_stairs"
    sandstone_wall = "sandstone_wall"
    scaffolding = "scaffolding"
    sculk = "sculk"
    sculk_catalyst = "sculk_catalyst"
    sculk_sensor = "sculk_sensor"
    sculk_shrieker = "sculk_shrieker"
    sculk_vein = "sculk_vein"
    scute = "scute"
    sea_lantern = "sea_lantern"
    sea_pickle = "sea_pickle"
    seagrass = "seagrass"
    sentry_armor_trim_smithing_template = "sentry_armor_trim_smithing_template"
    shaper_armor_trim_smithing_template = "shaper_armor_trim_smithing_template"
    sheaf_pottery_sherd = "sheaf_pottery_sherd"
    shears = "shears"
    sheep_spawn_egg = "sheep_spawn_egg"
    shelter_pottery_sherd = "shelter_pottery_sherd"
    shield = "shield"
    shroomlight = "shroomlight"
    shulker_box = "shulker_box"
    shulker_shell = "shulker_shell"
    shulker_spawn_egg = "shulker_spawn_egg"
    silence_armor_trim_smithing_template = "silence_armor_trim_smithing_template"
    silverfish_spawn_egg = "silverfish_spawn_egg"
    skeleton_horse_spawn_egg = "skeleton_horse_spawn_egg"
    skeleton_skull = "skeleton_skull"
    skeleton_spawn_egg = "skeleton_spawn_egg"
    skull_banner_pattern = "skull_banner_pattern"
    skull_pottery_sherd = "skull_pottery_sherd"
    slime_ball = "slime_ball"
    slime_block = "slime_block"
    slime_spawn_egg = "slime_spawn_egg"
    small_amethyst_bud = "small_amethyst_bud"
    small_dripleaf = "small_dripleaf"
    smithing_table = "smithing_table"
    smoker = "smoker"
    smooth_basalt = "smooth_basalt"
    smooth_quartz = "smooth_quartz"
    smooth_quartz_slab = "smooth_quartz_slab"
    smooth_quartz_stairs = "smooth_quartz_stairs"
    smooth_red_sandstone = "smooth_red_sandstone"
    smooth_red_sandstone_slab = "smooth_red_sandstone_slab"
    smooth_red_sandstone_stairs = "smooth_red_sandstone_stairs"
    smooth_sandstone = "smooth_sandstone"
    smooth_sandstone_slab = "smooth_sandstone_slab"
    smooth_sandstone_stairs = "smooth_sandstone_stairs"
    smooth_stone = "smooth_stone"
    smooth_stone_slab = "smooth_stone_slab"
    sniffer_egg = "sniffer_egg"
    sniffer_spawn_egg = "sniffer_spawn_egg"
    snort_pottery_sherd = "snort_pottery_sherd"
    snout_armor_trim_smithing_template = "snout_armor_trim_smithing_template"
    snow = "snow"
    snow_block = "snow_block"
    snow_golem_spawn_egg = "snow_golem_spawn_egg"
    snowball = "snowball"
    soul_campfire = "soul_campfire"
    soul_lantern = "soul_lantern"
    soul_sand = "soul_sand"
    soul_soil = "soul_soil"
    soul_torch = "soul_torch"
    spawner = "spawner"
    spectral_arrow = "spectral_arrow"
    spider_eye = "spider_eye"
    spider_spawn_egg = "spider_spawn_egg"
    spire_armor_trim_smithing_template = "spire_armor_trim_smithing_template"
    splash_potion = "splash_potion"
    sponge = "sponge"
    spore_blossom = "spore_blossom"
    spruce_boat = "spruce_boat"
    spruce_button = "spruce_button"
    spruce_chest_boat = "spruce_chest_boat"
    spruce_door = "spruce_door"
    spruce_fence = "spruce_fence"
    spruce_fence_gate = "spruce_fence_gate"
    spruce_hanging_sign = "spruce_hanging_sign"
    spruce_leaves = "spruce_leaves"
    spruce_log = "spruce_log"
    spruce_planks = "spruce_planks"
    spruce_pressure_plate = "spruce_pressure_plate"
    spruce_sapling = "spruce_sapling"
    spruce_sign = "spruce_sign"
    spruce_slab = "spruce_slab"
    spruce_stairs = "spruce_stairs"
    spruce_trapdoor = "spruce_trapdoor"
    spruce_wood = "spruce_wood"
    spyglass = "spyglass"
    squid_spawn_egg = "squid_spawn_egg"
    stick = "stick"
    sticky_piston = "sticky_piston"
    stone = "stone"
    stone_axe = "stone_axe"
    stone_brick_slab = "stone_brick_slab"
    stone_brick_stairs = "stone_brick_stairs"
    stone_brick_wall = "stone_brick_wall"
    stone_bricks = "stone_bricks"
    stone_button = "stone_button"
    stone_hoe = "stone_hoe"
    stone_pickaxe = "stone_pickaxe"
    stone_pressure_plate = "stone_pressure_plate"
    stone_shovel = "stone_shovel"
    stone_slab = "stone_slab"
    stone_stairs = "stone_stairs"
    stone_sword = "stone_sword"
    stonecutter = "stonecutter"
    stray_spawn_egg = "stray_spawn_egg"
    strider_spawn_egg = "strider_spawn_egg"
    string = "string"
    stripped_acacia_log = "stripped_acacia_log"
    stripped_acacia_wood = "stripped_acacia_wood"
    stripped_bamboo_block = "stripped_bamboo_block"
    stripped_birch_log = "stripped_birch_log"
    stripped_birch_wood = "stripped_birch_wood"
    stripped_cherry_log = "stripped_cherry_log"
    stripped_cherry_wood = "stripped_cherry_wood"
    stripped_crimson_hyphae = "stripped_crimson_hyphae"
    stripped_crimson_stem = "stripped_crimson_stem"
    stripped_dark_oak_log = "stripped_dark_oak_log"
    stripped_dark_oak_wood = "stripped_dark_oak_wood"
    stripped_jungle_log = "stripped_jungle_log"
    stripped_jungle_wood = "stripped_jungle_wood"
    stripped_mangrove_log = "stripped_mangrove_log"
    stripped_mangrove_wood = "stripped_mangrove_wood"
    stripped_oak_log = "stripped_oak_log"
    stripped_oak_wood = "stripped_oak_wood"
    stripped_spruce_log = "stripped_spruce_log"
    stripped_spruce_wood = "stripped_spruce_wood"
    stripped_warped_hyphae = "stripped_warped_hyphae"
    stripped_warped_stem = "stripped_warped_stem"
    structure_block = "structure_block"
    structure_void = "structure_void"
    sugar = "sugar"
    sugar_cane = "sugar_cane"
    sunflower = "sunflower"
    suspicious_gravel = "suspicious_gravel"
    suspicious_sand = "suspicious_sand"
    suspicious_stew = "suspicious_stew"
    sweet_berries = "sweet_berries"
    tadpole_bucket = "tadpole_bucket"
    tadpole_spawn_egg = "tadpole_spawn_egg"
    tall_grass = "tall_grass"
    target = "target"
    terracotta = "terracotta"
    tide_armor_trim_smithing_template = "tide_armor_trim_smithing_template"
    tinted_glass = "tinted_glass"
    tipped_arrow = "tipped_arrow"
    tnt = "tnt"
    tnt_minecart = "tnt_minecart"
    torch = "torch"
    torchflower = "torchflower"
    torchflower_seeds = "torchflower_seeds"
    totem_of_undying = "totem_of_undying"
    trader_llama_spawn_egg = "trader_llama_spawn_egg"
    trapped_chest = "trapped_chest"
    trident = "trident"
    tripwire_hook = "tripwire_hook"
    tropical_fish = "tropical_fish"
    tropical_fish_bucket = "tropical_fish_bucket"
    tropical_fish_spawn_egg = "tropical_fish_spawn_egg"
    tube_coral = "tube_coral"
    tube_coral_block = "tube_coral_block"
    tube_coral_fan = "tube_coral_fan"
    tuff = "tuff"
    turtle_egg = "turtle_egg"
    turtle_helmet = "turtle_helmet"
    turtle_spawn_egg = "turtle_spawn_egg"
    twisting_vines = "twisting_vines"
    verdant_froglight = "verdant_froglight"
    vex_armor_trim_smithing_template = "vex_armor_trim_smithing_template"
    vex_spawn_egg = "vex_spawn_egg"
    villager_spawn_egg = "villager_spawn_egg"
    vindicator_spawn_egg = "vindicator_spawn_egg"
    vine = "vine"
    wandering_trader_spawn_egg = "wandering_trader_spawn_egg"
    ward_armor_trim_smithing_template = "ward_armor_trim_smithing_template"
    warden_spawn_egg = "warden_spawn_egg"
    warped_button = "warped_button"
    warped_door = "warped_door"
    warped_fence = "warped_fence"
    warped_fence_gate = "warped_fence_gate"
    warped_fungus = "warped_fungus"
    warped_fungus_on_a_stick = "warped_fungus_on_a_stick"
    warped_hanging_sign = "warped_hanging_sign"
    warped_hyphae = "warped_hyphae"
    warped_nylium = "warped_nylium"
    warped_planks = "warped_planks"
    warped_pressure_plate = "warped_pressure_plate"
    warped_roots = "warped_roots"
    warped_sign = "warped_sign"
    warped_slab = "warped_slab"
    warped_stairs = "warped_stairs"
    warped_stem = "warped_stem"
    warped_trapdoor = "warped_trapdoor"
    warped_wart_block = "warped_wart_block"
    water_bucket = "water_bucket"
    waxed_copper_block = "waxed_copper_block"
    waxed_cut_copper = "waxed_cut_copper"
    waxed_cut_copper_slab = "waxed_cut_copper_slab"
    waxed_cut_copper_stairs = "waxed_cut_copper_stairs"
    waxed_exposed_copper = "waxed_exposed_copper"
    waxed_exposed_cut_copper = "waxed_exposed_cut_copper"
    waxed_exposed_cut_copper_slab = "waxed_exposed_cut_copper_slab"
    waxed_exposed_cut_copper_stairs = "waxed_exposed_cut_copper_stairs"
    waxed_oxidized_copper = "waxed_oxidized_copper"
    waxed_oxidized_cut_copper = "waxed_oxidized_cut_copper"
    waxed_oxidized_cut_copper_slab = "waxed_oxidized_cut_copper_slab"
    waxed_oxidized_cut_copper_stairs = "waxed_oxidized_cut_copper_stairs"
    waxed_weathered_copper = "waxed_weathered_copper"
    waxed_weathered_cut_copper = "waxed_weathered_cut_copper"
    waxed_weathered_cut_copper_slab = "waxed_weathered_cut_copper_slab"
    waxed_weathered_cut_copper_stairs = "waxed_weathered_cut_copper_stairs"
    wayfinder_armor_trim_smithing_template = "wayfinder_armor_trim_smithing_template"
    weathered_copper = "weathered_copper"
    weathered_cut_copper = "weathered_cut_copper"
    weathered_cut_copper_slab = "weathered_cut_copper_slab"
    weathered_cut_copper_stairs = "weathered_cut_copper_stairs"
    weeping_vines = "weeping_vines"
    wet_sponge = "wet_sponge"
    wheat = "wheat"
    wheat_seeds = "wheat_seeds"
    white_banner = "white_banner"
    white_bed = "white_bed"
    white_candle = "white_candle"
    white_carpet = "white_carpet"
    white_concrete = "white_concrete"
    white_concrete_powder = "white_concrete_powder"
    white_dye = "white_dye"
    white_glazed_terracotta = "white_glazed_terracotta"
    white_shulker_box = "white_shulker_box"
    white_stained_glass = "white_stained_glass"
    white_stained_glass_pane = "white_stained_glass_pane"
    white_terracotta = "white_terracotta"
    white_tulip = "white_tulip"
    white_wool = "white_wool"
    wild_armor_trim_smithing_template = "wild_armor_trim_smithing_template"
    witch_spawn_egg = "witch_spawn_egg"
    wither_rose = "wither_rose"
    wither_skeleton_skull = "wither_skeleton_skull"
    wither_skeleton_spawn_egg = "wither_skeleton_spawn_egg"
    wither_spawn_egg = "wither_spawn_egg"
    wolf_spawn_egg = "wolf_spawn_egg"
    wooden_axe = "wooden_axe"
    wooden_hoe = "wooden_hoe"
    wooden_pickaxe = "wooden_pickaxe"
    wooden_shovel = "wooden_shovel"
    wooden_sword = "wooden_sword"
    writable_book = "writable_book"
    written_book = "written_book"
    yellow_banner = "yellow_banner"
    yellow_bed = "yellow_bed"
    yellow_candle = "yellow_candle"
    yellow_carpet = "yellow_carpet"
    yellow_concrete = "yellow_concrete"
    yellow_concrete_powder = "yellow_concrete_powder"
    yellow_dye = "yellow_dye"
    yellow_glazed_terracotta = "yellow_glazed_terracotta"
    yellow_shulker_box = "yellow_shulker_box"
    yellow_stained_glass = "yellow_stained_glass"
    yellow_stained_glass_pane = "yellow_stained_glass_pane"
    yellow_terracotta = "yellow_terracotta"
    yellow_wool = "yellow_wool"
    zoglin_spawn_egg = "zoglin_spawn_egg"
    zombie_head = "zombie_head"
    zombie_horse_spawn_egg = "zombie_horse_spawn_egg"
    zombie_spawn_egg = "zombie_spawn_egg"
    zombie_villager_spawn_egg = "zombie_villager_spawn_egg"
    zombified_piglin_spawn_egg = "zombified_piglin_spawn_egg"



"""
Enum for all heightmaps in Minecraft.
"""
class Heightmap:
    world_surface = "world_surface"
    motion_blocking = "motion_blocking"
    motion_blocking_no_leaves = "motion_blocking_no_leaves"
    ocean_floor = "ocean_floor"



"""
Submodule for creating and managing recipes - shaped crafting, smelting, etc.
"""
from typing import TypeVar

from scute.internal.utils import _createJsonFile, formatText
from scute.tags import ItemTag

class Recipe:
    pass

class ShapedCraftingLayout:
    def __init__(self, row1: str, row2: str, row3: str, key: dict[str, str | ItemTag]):
        """
        A layout for a shaped crafting recipe
        Args:
            row1: A two or three letter string representing the first row of the grid
            row2: A two or three letter string representing the second row of the grid
            row3: A three letter string representing the third row of the grid - can be None for 2x2 grids
            key: A dictionary mapping the letters or symbols used in the rows to the items they represent. e.g. {"a": Item.cobblestone, "b": Item.dirt}
        """
        if row3 is not None:
            self.rows = [row1, row2, row3]
        else:
            self.rows = [row1, row2]
        self.key = key

class RecipeType:
    class blasting(Recipe):
        def __init__(self, input: str | ItemTag, output: str):
            """
            A blasting recipe, with one input and one output that accept no nbt
            Args:
                input: The input item, like Item.cobblestone
                output: The output item
            """
            self.input = input
            self.output = output
            self.json = _singleInOutRecipe("blasting", input, output)

    class campfire_cooking(Recipe):
        def __init__(self, input: str | ItemTag, output: str):
            """
            A campfire cooking recipe, with one input and one output that accept no nbt
            Args:
                input: The input item, like Item.cobblestone
                output: The output item
            """
            self.input = input
            self.output = output
            self.json = _singleInOutRecipe("campfire_cooking", input, output)

    class crafting_shaped(Recipe):
        def __init__(self, layout: ShapedCraftingLayout, output: str, count: int = 1):
            """
            A shaped crafting recipe in a crafting table, with a layout and an output that accept no nbt
            Args:
                layout: The items that need to be placed in the grid, and their locations
                output: The output item of the recipe
            """
            self.layout = layout
            self.output = output

            self.json = {
                "type": "minecraft:crafting_shaped",
                "pattern": layout.rows,
                "key": {
                    i: ({"tag": j.reference} if isinstance(j, ItemTag) else {"item": j})
                    for i, j in layout.key.items()
                },
                "result": {
                    "item": output,
                    "count": count
                }
            }

    class crafting_shapeless(Recipe):
        def __init__(self, ingredients: list[str | ItemTag], output: str, count: int = 1):
            """
            A shapeless crafting recipe in a crafting table, with a list of ingredients and an output that accept no nbt
            Args:
                ingredients: The items that need to be placed in the grid, and their locations
                output: The output item of the recipe
                count: The amount of output items
            """
            self.ingredients = ingredients
            self.output = output

            self.json = {
                "type": "minecraft:crafting_shapeless",
                "ingredients": [
                    {"tag": i.reference} if isinstance(i, ItemTag) else {"item": i}
                    for i in ingredients
                ],
                "result": {
                    "item": output,
                    "count": count
                }
            }

    class smelting(Recipe):
        def __init__(self, input: str | ItemTag, output: str):
            """
            A smelting recipe, with one input and one output that accept no nbt
            Args:
                input: The input item, like Item.cobblestone
                output: The output item
            """
            self.input = input
            self.output = output
            self.json = _singleInOutRecipe("smelting", input, output)

    class smithing(Recipe):
        """Unimplemented"""
        pass

    class smoking(Recipe):
        def __init__(self, input: str | ItemTag, output: str):
            """
            A smoking recipe, with one input and one output that accept no nbt
            Args:
                input: The input item, like Item.cobblestone
                output: The output item
            """
            self.input = input
            self.output = output
            self.json = _singleInOutRecipe("smoking", input, output)

    class stonecutting(Recipe):
        def __init__(self, input: str | ItemTag, output: str, count: int = 1):
            """
            A stonecutting recipe, with one input and an output that accept no nbt
            Args:
                input: The input item, like Item.cobblestone
                output: The output item
                count: The amount of output items
            """
            self.input = input
            self.output = output
            self.count = count
            self.json = _singleInOutRecipe("smoking", input, output, count=count)

def _singleInOutRecipe(id, input, output, count=None):
    out = \
        {
            "type": f"minecraft:{id}",
            "ingredient": {
                "item": input
            },
            "result": output
        }\
        if isinstance(input, str) else\
        {
            "type": f"minecraft:{id}",
            "ingredient": {
                "tag": input.reference
            },
            "result": output
        }
    if count is not None and count != 1:
        out["count"] = count

    return out


_Recipe = TypeVar('_Recipe', bound=Recipe)

def registerRecipe(recipe: _Recipe, namespace: str, name: str):
    _createJsonFile(namespace, name, r"recipes", recipe.json)
    print(formatText(f"Created recipe {namespace}:{name}", 32))



"""
Enum for all relations in the game.
"""
class Relation:
    attacker = "attacker"
    controller = "controller"
    leasher = "leasher"
    origin = "origin"
    owner = "owner"
    passengers = "passengers"
    target = "target"
    vehicle = "vehicle"



"""
Submodule for json-formatted text, used in /tellraw or item names etc
"""
from scute import _JsonText
from scute.internal.dictToNBT import dictToNBT
from scute.items import Item

def JSONText(string: str,
             italic: bool = False,
             colour: str = None,
             strikethrough: bool = None,
             bold: bool = None,
             underlined: bool = None,
             obfuscated: bool = None,
             click_url: str = None,
             click_command: str = None,
             click_suggest: str = None,
             click_clipboard: str = None,
             hover_text: dict | str = None,
             hover_item: Item = None,
             hover_entity: dict = None,
             ):
    """
    Creates json-formatted text. A maximum of one of the click_* or hover_* arguments can be specified
    Args:
        string: The actual text to show
        italic: Bool. Whether or not to make the text italic
        colour: The colour of the text. E.g: `Colour.red`, `"red"`, `"#FFFFFF"`, or `Colour.hex("#FFFFFF)`
        strikethrough: Bool. Whether or not to make the text struckthrough
        bold: Bool. Whether or not to make the text bold
        underlined: Bool. Whether or not to make the text underlined
        obfuscated: Bool. Whether or not to make the text obfuscated
        click_url: The URL opened when the text is clicked - must include schema
        click_command: The command run by the user when the text is clicked
        click_suggest: The string added to the chat window when the text is clicked
        click_clipboard: The value that is copied to the clipboard when the text is clicked
        hover_text: The json-formatted text that is shown when the text is hovered (or just a string)
        hover_item: The item that is shown when the text is hovered
        hover_entity: The entity that is shown when the text is hovered - e.g. {"type":"pig", "id":"0-0-0-0"}. json-formatted `name` field is optional.
    Returns:
        A JsonText instance, to be passed into various functions
    """
    optionalArgs = {
        "strikethrough": strikethrough,
        "color": colour,
        "bold": bold,
        "underlined": underlined,
        "obfuscated": obfuscated
    }

    text = {
        "text": string,
        "italic": italic,
        **{key: value for key, value in optionalArgs.items() if value is not None}
    }

    if click_url is not None: text["clickEvent"] = {"action": "open_url", "value": click_url}
    if click_command is not None: text["clickEvent"] = {"action": "run_command", "value": click_command}
    if click_suggest is not None: text["clickEvent"] = {"action": "suggest_command", "value": click_suggest}
    if click_clipboard is not None: text["clickEvent"] = {"action": "copy_to_clipboard", "value": click_clipboard}
    if hover_text is not None: text["hoverEvent"] = {"action": "show_text", "contents": hover_text}
    if hover_item is not None:
        data = {
            "action": "show_item",
            "contents": {
                "id": hover_item.id,
                "count": hover_item.count,
            }
        }
        if hover_item.nbt is not None:
            data["contents"]["tag"] = dictToNBT(hover_item.nbt)
        text["hoverEvent"] = data
    if hover_entity is not None:
        if "type" in hover_entity:
            if "id" in hover_entity:
                data = {
                    "action": "show_entity",
                    "contents": {
                        "type": hover_entity["type"],
                        "id": hover_entity["id"],
                    }
                }
                if "name" in hover_entity:
                    data["contents"]["name"] = hover_entity["name"]
                text["hoverEvent"] = data
            else:
                raise Exception("You must specify the UUID of the entity to show in a `show_entity` hover event, under the `id` field")
        else:
            raise Exception(
                "You must specify the type of the entity to show in a `show_entity` hover event, under the `type` field")

    return _JsonText(text)


class Colour:
    """
    An enum of all the colours in the base game. Colour.red is the exact same as "red"
    """
    black = "black"
    dark_blue = "dark_blue"
    dark_green = "dark_green"
    dark_aqua = "dark_aqua"
    dark_red = "dark_red"
    dark_purple = "dark_purple"
    gold = "gold"
    gray = "gray"
    dark_gray = "gray"
    grey = "gray"
    dark_grey = "gray"
    blue = "blue"
    green = "green"
    aqua = "aqua"
    red = "red"
    light_purple = "light_purple"
    yellow = "yellow"
    white = "white"

    @staticmethod
    def hex(hex: str):
        """
        A custom colour - alternatively, just use a string like "#FFFFFF"
        """
        return hex



"""
Various data types used in NBT.
"""
class _NumberType:
    letter = None
    int = False

    def __init__(self, number):
        if self.int:
            self.number = int(number)
        else:
            self.number = number

    def getNbt(self):
        return str(self.number) + self.letter


class Byte(_NumberType):
    letter = "b"
    int = True


class Long(_NumberType):
    letter = "l"
    int = True


class Short(_NumberType):
    letter = "s"
    int = True


class Float(_NumberType):
    letter = "f"


class Double(_NumberType):
    letter = "d"



"""
Enum and class for blocks.
"""
class Block:
    def __init__(self, name: str, nbt: dict = None):
        self.id = name
        self.nbt = nbt

    acacia_button = "acacia_button"
    acacia_door = "acacia_door"
    acacia_fence = "acacia_fence"
    acacia_fence_gate = "acacia_fence_gate"
    acacia_hanging_sign = "acacia_hanging_sign"
    acacia_leaves = "acacia_leaves"
    acacia_log = "acacia_log"
    acacia_planks = "acacia_planks"
    acacia_pressure_plate = "acacia_pressure_plate"
    acacia_sapling = "acacia_sapling"
    acacia_sign = "acacia_sign"
    acacia_slab = "acacia_slab"
    acacia_stairs = "acacia_stairs"
    acacia_trapdoor = "acacia_trapdoor"
    acacia_wall_hanging_sign = "acacia_wall_hanging_sign"
    acacia_wall_sign = "acacia_wall_sign"
    acacia_wood = "acacia_wood"
    activator_rail = "activator_rail"
    air = "air"
    allium = "allium"
    amethyst_block = "amethyst_block"
    amethyst_cluster = "amethyst_cluster"
    ancient_debris = "ancient_debris"
    andesite = "andesite"
    andesite_slab = "andesite_slab"
    andesite_stairs = "andesite_stairs"
    andesite_wall = "andesite_wall"
    anvil = "anvil"
    attached_melon_stem = "attached_melon_stem"
    attached_pumpkin_stem = "attached_pumpkin_stem"
    azalea = "azalea"
    azalea_leaves = "azalea_leaves"
    azure_bluet = "azure_bluet"
    bamboo = "bamboo"
    bamboo_block = "bamboo_block"
    bamboo_button = "bamboo_button"
    bamboo_door = "bamboo_door"
    bamboo_fence = "bamboo_fence"
    bamboo_fence_gate = "bamboo_fence_gate"
    bamboo_hanging_sign = "bamboo_hanging_sign"
    bamboo_mosaic = "bamboo_mosaic"
    bamboo_mosaic_slab = "bamboo_mosaic_slab"
    bamboo_mosaic_stairs = "bamboo_mosaic_stairs"
    bamboo_planks = "bamboo_planks"
    bamboo_pressure_plate = "bamboo_pressure_plate"
    bamboo_sapling = "bamboo_sapling"
    bamboo_sign = "bamboo_sign"
    bamboo_slab = "bamboo_slab"
    bamboo_stairs = "bamboo_stairs"
    bamboo_trapdoor = "bamboo_trapdoor"
    bamboo_wall_hanging_sign = "bamboo_wall_hanging_sign"
    bamboo_wall_sign = "bamboo_wall_sign"
    barrel = "barrel"
    barrier = "barrier"
    basalt = "basalt"
    beacon = "beacon"
    bedrock = "bedrock"
    bee_nest = "bee_nest"
    beehive = "beehive"
    beetroots = "beetroots"
    bell = "bell"
    big_dripleaf = "big_dripleaf"
    big_dripleaf_stem = "big_dripleaf_stem"
    birch_button = "birch_button"
    birch_door = "birch_door"
    birch_fence = "birch_fence"
    birch_fence_gate = "birch_fence_gate"
    birch_hanging_sign = "birch_hanging_sign"
    birch_leaves = "birch_leaves"
    birch_log = "birch_log"
    birch_planks = "birch_planks"
    birch_pressure_plate = "birch_pressure_plate"
    birch_sapling = "birch_sapling"
    birch_sign = "birch_sign"
    birch_slab = "birch_slab"
    birch_stairs = "birch_stairs"
    birch_trapdoor = "birch_trapdoor"
    birch_wall_hanging_sign = "birch_wall_hanging_sign"
    birch_wall_sign = "birch_wall_sign"
    birch_wood = "birch_wood"
    black_banner = "black_banner"
    black_bed = "black_bed"
    black_candle = "black_candle"
    black_candle_cake = "black_candle_cake"
    black_carpet = "black_carpet"
    black_concrete = "black_concrete"
    black_concrete_powder = "black_concrete_powder"
    black_glazed_terracotta = "black_glazed_terracotta"
    black_shulker_box = "black_shulker_box"
    black_stained_glass = "black_stained_glass"
    black_stained_glass_pane = "black_stained_glass_pane"
    black_terracotta = "black_terracotta"
    black_wall_banner = "black_wall_banner"
    black_wool = "black_wool"
    blackstone = "blackstone"
    blackstone_slab = "blackstone_slab"
    blackstone_stairs = "blackstone_stairs"
    blackstone_wall = "blackstone_wall"
    blast_furnace = "blast_furnace"
    blue_banner = "blue_banner"
    blue_bed = "blue_bed"
    blue_candle = "blue_candle"
    blue_candle_cake = "blue_candle_cake"
    blue_carpet = "blue_carpet"
    blue_concrete = "blue_concrete"
    blue_concrete_powder = "blue_concrete_powder"
    blue_glazed_terracotta = "blue_glazed_terracotta"
    blue_ice = "blue_ice"
    blue_orchid = "blue_orchid"
    blue_shulker_box = "blue_shulker_box"
    blue_stained_glass = "blue_stained_glass"
    blue_stained_glass_pane = "blue_stained_glass_pane"
    blue_terracotta = "blue_terracotta"
    blue_wall_banner = "blue_wall_banner"
    blue_wool = "blue_wool"
    bone_block = "bone_block"
    bookshelf = "bookshelf"
    brain_coral = "brain_coral"
    brain_coral_block = "brain_coral_block"
    brain_coral_fan = "brain_coral_fan"
    brain_coral_wall_fan = "brain_coral_wall_fan"
    brewing_stand = "brewing_stand"
    brick_slab = "brick_slab"
    brick_stairs = "brick_stairs"
    brick_wall = "brick_wall"
    bricks = "bricks"
    brown_banner = "brown_banner"
    brown_bed = "brown_bed"
    brown_candle = "brown_candle"
    brown_candle_cake = "brown_candle_cake"
    brown_carpet = "brown_carpet"
    brown_concrete = "brown_concrete"
    brown_concrete_powder = "brown_concrete_powder"
    brown_glazed_terracotta = "brown_glazed_terracotta"
    brown_mushroom = "brown_mushroom"
    brown_mushroom_block = "brown_mushroom_block"
    brown_shulker_box = "brown_shulker_box"
    brown_stained_glass = "brown_stained_glass"
    brown_stained_glass_pane = "brown_stained_glass_pane"
    brown_terracotta = "brown_terracotta"
    brown_wall_banner = "brown_wall_banner"
    brown_wool = "brown_wool"
    bubble_column = "bubble_column"
    bubble_coral = "bubble_coral"
    bubble_coral_block = "bubble_coral_block"
    bubble_coral_fan = "bubble_coral_fan"
    bubble_coral_wall_fan = "bubble_coral_wall_fan"
    budding_amethyst = "budding_amethyst"
    cactus = "cactus"
    cake = "cake"
    calcite = "calcite"
    calibrated_sculk_sensor = "calibrated_sculk_sensor"
    campfire = "campfire"
    candle = "candle"
    candle_cake = "candle_cake"
    carrots = "carrots"
    cartography_table = "cartography_table"
    carved_pumpkin = "carved_pumpkin"
    cauldron = "cauldron"
    cave_air = "cave_air"
    cave_vines = "cave_vines"
    cave_vines_plant = "cave_vines_plant"
    chain = "chain"
    chain_command_block = "chain_command_block"
    cherry_button = "cherry_button"
    cherry_door = "cherry_door"
    cherry_fence = "cherry_fence"
    cherry_fence_gate = "cherry_fence_gate"
    cherry_hanging_sign = "cherry_hanging_sign"
    cherry_leaves = "cherry_leaves"
    cherry_log = "cherry_log"
    cherry_planks = "cherry_planks"
    cherry_pressure_plate = "cherry_pressure_plate"
    cherry_sapling = "cherry_sapling"
    cherry_sign = "cherry_sign"
    cherry_slab = "cherry_slab"
    cherry_stairs = "cherry_stairs"
    cherry_trapdoor = "cherry_trapdoor"
    cherry_wall_hanging_sign = "cherry_wall_hanging_sign"
    cherry_wall_sign = "cherry_wall_sign"
    cherry_wood = "cherry_wood"
    chest = "chest"
    chipped_anvil = "chipped_anvil"
    chiseled_bookshelf = "chiseled_bookshelf"
    chiseled_deepslate = "chiseled_deepslate"
    chiseled_nether_bricks = "chiseled_nether_bricks"
    chiseled_polished_blackstone = "chiseled_polished_blackstone"
    chiseled_quartz_block = "chiseled_quartz_block"
    chiseled_red_sandstone = "chiseled_red_sandstone"
    chiseled_sandstone = "chiseled_sandstone"
    chiseled_stone_bricks = "chiseled_stone_bricks"
    chorus_flower = "chorus_flower"
    chorus_plant = "chorus_plant"
    clay = "clay"
    coal_block = "coal_block"
    coal_ore = "coal_ore"
    coarse_dirt = "coarse_dirt"
    cobbled_deepslate = "cobbled_deepslate"
    cobbled_deepslate_slab = "cobbled_deepslate_slab"
    cobbled_deepslate_stairs = "cobbled_deepslate_stairs"
    cobbled_deepslate_wall = "cobbled_deepslate_wall"
    cobblestone = "cobblestone"
    cobblestone_slab = "cobblestone_slab"
    cobblestone_stairs = "cobblestone_stairs"
    cobblestone_wall = "cobblestone_wall"
    cobweb = "cobweb"
    cocoa = "cocoa"
    command_block = "command_block"
    comparator = "comparator"
    composter = "composter"
    conduit = "conduit"
    copper_block = "copper_block"
    copper_ore = "copper_ore"
    cornflower = "cornflower"
    cracked_deepslate_bricks = "cracked_deepslate_bricks"
    cracked_deepslate_tiles = "cracked_deepslate_tiles"
    cracked_nether_bricks = "cracked_nether_bricks"
    cracked_polished_blackstone_bricks = "cracked_polished_blackstone_bricks"
    cracked_stone_bricks = "cracked_stone_bricks"
    crafting_table = "crafting_table"
    creeper_head = "creeper_head"
    creeper_wall_head = "creeper_wall_head"
    crimson_button = "crimson_button"
    crimson_door = "crimson_door"
    crimson_fence = "crimson_fence"
    crimson_fence_gate = "crimson_fence_gate"
    crimson_fungus = "crimson_fungus"
    crimson_hanging_sign = "crimson_hanging_sign"
    crimson_hyphae = "crimson_hyphae"
    crimson_nylium = "crimson_nylium"
    crimson_planks = "crimson_planks"
    crimson_pressure_plate = "crimson_pressure_plate"
    crimson_roots = "crimson_roots"
    crimson_sign = "crimson_sign"
    crimson_slab = "crimson_slab"
    crimson_stairs = "crimson_stairs"
    crimson_stem = "crimson_stem"
    crimson_trapdoor = "crimson_trapdoor"
    crimson_wall_hanging_sign = "crimson_wall_hanging_sign"
    crimson_wall_sign = "crimson_wall_sign"
    crying_obsidian = "crying_obsidian"
    cut_copper = "cut_copper"
    cut_copper_slab = "cut_copper_slab"
    cut_copper_stairs = "cut_copper_stairs"
    cut_red_sandstone = "cut_red_sandstone"
    cut_red_sandstone_slab = "cut_red_sandstone_slab"
    cut_sandstone = "cut_sandstone"
    cut_sandstone_slab = "cut_sandstone_slab"
    cyan_banner = "cyan_banner"
    cyan_bed = "cyan_bed"
    cyan_candle = "cyan_candle"
    cyan_candle_cake = "cyan_candle_cake"
    cyan_carpet = "cyan_carpet"
    cyan_concrete = "cyan_concrete"
    cyan_concrete_powder = "cyan_concrete_powder"
    cyan_glazed_terracotta = "cyan_glazed_terracotta"
    cyan_shulker_box = "cyan_shulker_box"
    cyan_stained_glass = "cyan_stained_glass"
    cyan_stained_glass_pane = "cyan_stained_glass_pane"
    cyan_terracotta = "cyan_terracotta"
    cyan_wall_banner = "cyan_wall_banner"
    cyan_wool = "cyan_wool"
    damaged_anvil = "damaged_anvil"
    dandelion = "dandelion"
    dark_oak_button = "dark_oak_button"
    dark_oak_door = "dark_oak_door"
    dark_oak_fence = "dark_oak_fence"
    dark_oak_fence_gate = "dark_oak_fence_gate"
    dark_oak_hanging_sign = "dark_oak_hanging_sign"
    dark_oak_leaves = "dark_oak_leaves"
    dark_oak_log = "dark_oak_log"
    dark_oak_planks = "dark_oak_planks"
    dark_oak_pressure_plate = "dark_oak_pressure_plate"
    dark_oak_sapling = "dark_oak_sapling"
    dark_oak_sign = "dark_oak_sign"
    dark_oak_slab = "dark_oak_slab"
    dark_oak_stairs = "dark_oak_stairs"
    dark_oak_trapdoor = "dark_oak_trapdoor"
    dark_oak_wall_hanging_sign = "dark_oak_wall_hanging_sign"
    dark_oak_wall_sign = "dark_oak_wall_sign"
    dark_oak_wood = "dark_oak_wood"
    dark_prismarine = "dark_prismarine"
    dark_prismarine_slab = "dark_prismarine_slab"
    dark_prismarine_stairs = "dark_prismarine_stairs"
    daylight_detector = "daylight_detector"
    dead_brain_coral = "dead_brain_coral"
    dead_brain_coral_block = "dead_brain_coral_block"
    dead_brain_coral_fan = "dead_brain_coral_fan"
    dead_brain_coral_wall_fan = "dead_brain_coral_wall_fan"
    dead_bubble_coral = "dead_bubble_coral"
    dead_bubble_coral_block = "dead_bubble_coral_block"
    dead_bubble_coral_fan = "dead_bubble_coral_fan"
    dead_bubble_coral_wall_fan = "dead_bubble_coral_wall_fan"
    dead_bush = "dead_bush"
    dead_fire_coral = "dead_fire_coral"
    dead_fire_coral_block = "dead_fire_coral_block"
    dead_fire_coral_fan = "dead_fire_coral_fan"
    dead_fire_coral_wall_fan = "dead_fire_coral_wall_fan"
    dead_horn_coral = "dead_horn_coral"
    dead_horn_coral_block = "dead_horn_coral_block"
    dead_horn_coral_fan = "dead_horn_coral_fan"
    dead_horn_coral_wall_fan = "dead_horn_coral_wall_fan"
    dead_tube_coral = "dead_tube_coral"
    dead_tube_coral_block = "dead_tube_coral_block"
    dead_tube_coral_fan = "dead_tube_coral_fan"
    dead_tube_coral_wall_fan = "dead_tube_coral_wall_fan"
    decorated_pot = "decorated_pot"
    deepslate = "deepslate"
    deepslate_brick_slab = "deepslate_brick_slab"
    deepslate_brick_stairs = "deepslate_brick_stairs"
    deepslate_brick_wall = "deepslate_brick_wall"
    deepslate_bricks = "deepslate_bricks"
    deepslate_coal_ore = "deepslate_coal_ore"
    deepslate_copper_ore = "deepslate_copper_ore"
    deepslate_diamond_ore = "deepslate_diamond_ore"
    deepslate_emerald_ore = "deepslate_emerald_ore"
    deepslate_gold_ore = "deepslate_gold_ore"
    deepslate_iron_ore = "deepslate_iron_ore"
    deepslate_lapis_ore = "deepslate_lapis_ore"
    deepslate_redstone_ore = "deepslate_redstone_ore"
    deepslate_tile_slab = "deepslate_tile_slab"
    deepslate_tile_stairs = "deepslate_tile_stairs"
    deepslate_tile_wall = "deepslate_tile_wall"
    deepslate_tiles = "deepslate_tiles"
    detector_rail = "detector_rail"
    diamond_block = "diamond_block"
    diamond_ore = "diamond_ore"
    diorite = "diorite"
    diorite_slab = "diorite_slab"
    diorite_stairs = "diorite_stairs"
    diorite_wall = "diorite_wall"
    dirt = "dirt"
    dirt_path = "dirt_path"
    dispenser = "dispenser"
    dragon_egg = "dragon_egg"
    dragon_head = "dragon_head"
    dragon_wall_head = "dragon_wall_head"
    dried_kelp_block = "dried_kelp_block"
    dripstone_block = "dripstone_block"
    dropper = "dropper"
    emerald_block = "emerald_block"
    emerald_ore = "emerald_ore"
    enchanting_table = "enchanting_table"
    end_gateway = "end_gateway"
    end_portal = "end_portal"
    end_portal_frame = "end_portal_frame"
    end_rod = "end_rod"
    end_stone = "end_stone"
    end_stone_brick_slab = "end_stone_brick_slab"
    end_stone_brick_stairs = "end_stone_brick_stairs"
    end_stone_brick_wall = "end_stone_brick_wall"
    end_stone_bricks = "end_stone_bricks"
    ender_chest = "ender_chest"
    exposed_copper = "exposed_copper"
    exposed_cut_copper = "exposed_cut_copper"
    exposed_cut_copper_slab = "exposed_cut_copper_slab"
    exposed_cut_copper_stairs = "exposed_cut_copper_stairs"
    farmland = "farmland"
    fern = "fern"
    fire = "fire"
    fire_coral = "fire_coral"
    fire_coral_block = "fire_coral_block"
    fire_coral_fan = "fire_coral_fan"
    fire_coral_wall_fan = "fire_coral_wall_fan"
    fletching_table = "fletching_table"
    flower_pot = "flower_pot"
    flowering_azalea = "flowering_azalea"
    flowering_azalea_leaves = "flowering_azalea_leaves"
    frogspawn = "frogspawn"
    frosted_ice = "frosted_ice"
    furnace = "furnace"
    gilded_blackstone = "gilded_blackstone"
    glass = "glass"
    glass_pane = "glass_pane"
    glow_lichen = "glow_lichen"
    glowstone = "glowstone"
    gold_block = "gold_block"
    gold_ore = "gold_ore"
    granite = "granite"
    granite_slab = "granite_slab"
    granite_stairs = "granite_stairs"
    granite_wall = "granite_wall"
    grass = "grass"
    grass_block = "grass_block"
    gravel = "gravel"
    gray_banner = "gray_banner"
    gray_bed = "gray_bed"
    gray_candle = "gray_candle"
    gray_candle_cake = "gray_candle_cake"
    gray_carpet = "gray_carpet"
    gray_concrete = "gray_concrete"
    gray_concrete_powder = "gray_concrete_powder"
    gray_glazed_terracotta = "gray_glazed_terracotta"
    gray_shulker_box = "gray_shulker_box"
    gray_stained_glass = "gray_stained_glass"
    gray_stained_glass_pane = "gray_stained_glass_pane"
    gray_terracotta = "gray_terracotta"
    gray_wall_banner = "gray_wall_banner"
    gray_wool = "gray_wool"
    green_banner = "green_banner"
    green_bed = "green_bed"
    green_candle = "green_candle"
    green_candle_cake = "green_candle_cake"
    green_carpet = "green_carpet"
    green_concrete = "green_concrete"
    green_concrete_powder = "green_concrete_powder"
    green_glazed_terracotta = "green_glazed_terracotta"
    green_shulker_box = "green_shulker_box"
    green_stained_glass = "green_stained_glass"
    green_stained_glass_pane = "green_stained_glass_pane"
    green_terracotta = "green_terracotta"
    green_wall_banner = "green_wall_banner"
    green_wool = "green_wool"
    grindstone = "grindstone"
    hanging_roots = "hanging_roots"
    hay_block = "hay_block"
    heavy_weighted_pressure_plate = "heavy_weighted_pressure_plate"
    honey_block = "honey_block"
    honeycomb_block = "honeycomb_block"
    hopper = "hopper"
    horn_coral = "horn_coral"
    horn_coral_block = "horn_coral_block"
    horn_coral_fan = "horn_coral_fan"
    horn_coral_wall_fan = "horn_coral_wall_fan"
    ice = "ice"
    infested_chiseled_stone_bricks = "infested_chiseled_stone_bricks"
    infested_cobblestone = "infested_cobblestone"
    infested_cracked_stone_bricks = "infested_cracked_stone_bricks"
    infested_deepslate = "infested_deepslate"
    infested_mossy_stone_bricks = "infested_mossy_stone_bricks"
    infested_stone = "infested_stone"
    infested_stone_bricks = "infested_stone_bricks"
    iron_bars = "iron_bars"
    iron_block = "iron_block"
    iron_door = "iron_door"
    iron_ore = "iron_ore"
    iron_trapdoor = "iron_trapdoor"
    jack_o_lantern = "jack_o_lantern"
    jigsaw = "jigsaw"
    jukebox = "jukebox"
    jungle_button = "jungle_button"
    jungle_door = "jungle_door"
    jungle_fence = "jungle_fence"
    jungle_fence_gate = "jungle_fence_gate"
    jungle_hanging_sign = "jungle_hanging_sign"
    jungle_leaves = "jungle_leaves"
    jungle_log = "jungle_log"
    jungle_planks = "jungle_planks"
    jungle_pressure_plate = "jungle_pressure_plate"
    jungle_sapling = "jungle_sapling"
    jungle_sign = "jungle_sign"
    jungle_slab = "jungle_slab"
    jungle_stairs = "jungle_stairs"
    jungle_trapdoor = "jungle_trapdoor"
    jungle_wall_hanging_sign = "jungle_wall_hanging_sign"
    jungle_wall_sign = "jungle_wall_sign"
    jungle_wood = "jungle_wood"
    kelp = "kelp"
    kelp_plant = "kelp_plant"
    ladder = "ladder"
    lantern = "lantern"
    lapis_block = "lapis_block"
    lapis_ore = "lapis_ore"
    large_amethyst_bud = "large_amethyst_bud"
    large_fern = "large_fern"
    lava = "lava"
    lava_cauldron = "lava_cauldron"
    lectern = "lectern"
    lever = "lever"
    light = "light"
    light_blue_banner = "light_blue_banner"
    light_blue_bed = "light_blue_bed"
    light_blue_candle = "light_blue_candle"
    light_blue_candle_cake = "light_blue_candle_cake"
    light_blue_carpet = "light_blue_carpet"
    light_blue_concrete = "light_blue_concrete"
    light_blue_concrete_powder = "light_blue_concrete_powder"
    light_blue_glazed_terracotta = "light_blue_glazed_terracotta"
    light_blue_shulker_box = "light_blue_shulker_box"
    light_blue_stained_glass = "light_blue_stained_glass"
    light_blue_stained_glass_pane = "light_blue_stained_glass_pane"
    light_blue_terracotta = "light_blue_terracotta"
    light_blue_wall_banner = "light_blue_wall_banner"
    light_blue_wool = "light_blue_wool"
    light_gray_banner = "light_gray_banner"
    light_gray_bed = "light_gray_bed"
    light_gray_candle = "light_gray_candle"
    light_gray_candle_cake = "light_gray_candle_cake"
    light_gray_carpet = "light_gray_carpet"
    light_gray_concrete = "light_gray_concrete"
    light_gray_concrete_powder = "light_gray_concrete_powder"
    light_gray_glazed_terracotta = "light_gray_glazed_terracotta"
    light_gray_shulker_box = "light_gray_shulker_box"
    light_gray_stained_glass = "light_gray_stained_glass"
    light_gray_stained_glass_pane = "light_gray_stained_glass_pane"
    light_gray_terracotta = "light_gray_terracotta"
    light_gray_wall_banner = "light_gray_wall_banner"
    light_gray_wool = "light_gray_wool"
    light_weighted_pressure_plate = "light_weighted_pressure_plate"
    lightning_rod = "lightning_rod"
    lilac = "lilac"
    lily_of_the_valley = "lily_of_the_valley"
    lily_pad = "lily_pad"
    lime_banner = "lime_banner"
    lime_bed = "lime_bed"
    lime_candle = "lime_candle"
    lime_candle_cake = "lime_candle_cake"
    lime_carpet = "lime_carpet"
    lime_concrete = "lime_concrete"
    lime_concrete_powder = "lime_concrete_powder"
    lime_glazed_terracotta = "lime_glazed_terracotta"
    lime_shulker_box = "lime_shulker_box"
    lime_stained_glass = "lime_stained_glass"
    lime_stained_glass_pane = "lime_stained_glass_pane"
    lime_terracotta = "lime_terracotta"
    lime_wall_banner = "lime_wall_banner"
    lime_wool = "lime_wool"
    lodestone = "lodestone"
    loom = "loom"
    magenta_banner = "magenta_banner"
    magenta_bed = "magenta_bed"
    magenta_candle = "magenta_candle"
    magenta_candle_cake = "magenta_candle_cake"
    magenta_carpet = "magenta_carpet"
    magenta_concrete = "magenta_concrete"
    magenta_concrete_powder = "magenta_concrete_powder"
    magenta_glazed_terracotta = "magenta_glazed_terracotta"
    magenta_shulker_box = "magenta_shulker_box"
    magenta_stained_glass = "magenta_stained_glass"
    magenta_stained_glass_pane = "magenta_stained_glass_pane"
    magenta_terracotta = "magenta_terracotta"
    magenta_wall_banner = "magenta_wall_banner"
    magenta_wool = "magenta_wool"
    magma_block = "magma_block"
    mangrove_button = "mangrove_button"
    mangrove_door = "mangrove_door"
    mangrove_fence = "mangrove_fence"
    mangrove_fence_gate = "mangrove_fence_gate"
    mangrove_hanging_sign = "mangrove_hanging_sign"
    mangrove_leaves = "mangrove_leaves"
    mangrove_log = "mangrove_log"
    mangrove_planks = "mangrove_planks"
    mangrove_pressure_plate = "mangrove_pressure_plate"
    mangrove_propagule = "mangrove_propagule"
    mangrove_roots = "mangrove_roots"
    mangrove_sign = "mangrove_sign"
    mangrove_slab = "mangrove_slab"
    mangrove_stairs = "mangrove_stairs"
    mangrove_trapdoor = "mangrove_trapdoor"
    mangrove_wall_hanging_sign = "mangrove_wall_hanging_sign"
    mangrove_wall_sign = "mangrove_wall_sign"
    mangrove_wood = "mangrove_wood"
    medium_amethyst_bud = "medium_amethyst_bud"
    melon = "melon"
    melon_stem = "melon_stem"
    moss_block = "moss_block"
    moss_carpet = "moss_carpet"
    mossy_cobblestone = "mossy_cobblestone"
    mossy_cobblestone_slab = "mossy_cobblestone_slab"
    mossy_cobblestone_stairs = "mossy_cobblestone_stairs"
    mossy_cobblestone_wall = "mossy_cobblestone_wall"
    mossy_stone_brick_slab = "mossy_stone_brick_slab"
    mossy_stone_brick_stairs = "mossy_stone_brick_stairs"
    mossy_stone_brick_wall = "mossy_stone_brick_wall"
    mossy_stone_bricks = "mossy_stone_bricks"
    moving_piston = "moving_piston"
    mud = "mud"
    mud_brick_slab = "mud_brick_slab"
    mud_brick_stairs = "mud_brick_stairs"
    mud_brick_wall = "mud_brick_wall"
    mud_bricks = "mud_bricks"
    muddy_mangrove_roots = "muddy_mangrove_roots"
    mushroom_stem = "mushroom_stem"
    mycelium = "mycelium"
    nether_brick_fence = "nether_brick_fence"
    nether_brick_slab = "nether_brick_slab"
    nether_brick_stairs = "nether_brick_stairs"
    nether_brick_wall = "nether_brick_wall"
    nether_bricks = "nether_bricks"
    nether_gold_ore = "nether_gold_ore"
    nether_portal = "nether_portal"
    nether_quartz_ore = "nether_quartz_ore"
    nether_sprouts = "nether_sprouts"
    nether_wart = "nether_wart"
    nether_wart_block = "nether_wart_block"
    netherite_block = "netherite_block"
    netherrack = "netherrack"
    note_block = "note_block"
    oak_button = "oak_button"
    oak_door = "oak_door"
    oak_fence = "oak_fence"
    oak_fence_gate = "oak_fence_gate"
    oak_hanging_sign = "oak_hanging_sign"
    oak_leaves = "oak_leaves"
    oak_log = "oak_log"
    oak_planks = "oak_planks"
    oak_pressure_plate = "oak_pressure_plate"
    oak_sapling = "oak_sapling"
    oak_sign = "oak_sign"
    oak_slab = "oak_slab"
    oak_stairs = "oak_stairs"
    oak_trapdoor = "oak_trapdoor"
    oak_wall_hanging_sign = "oak_wall_hanging_sign"
    oak_wall_sign = "oak_wall_sign"
    oak_wood = "oak_wood"
    observer = "observer"
    obsidian = "obsidian"
    ochre_froglight = "ochre_froglight"
    orange_banner = "orange_banner"
    orange_bed = "orange_bed"
    orange_candle = "orange_candle"
    orange_candle_cake = "orange_candle_cake"
    orange_carpet = "orange_carpet"
    orange_concrete = "orange_concrete"
    orange_concrete_powder = "orange_concrete_powder"
    orange_glazed_terracotta = "orange_glazed_terracotta"
    orange_shulker_box = "orange_shulker_box"
    orange_stained_glass = "orange_stained_glass"
    orange_stained_glass_pane = "orange_stained_glass_pane"
    orange_terracotta = "orange_terracotta"
    orange_tulip = "orange_tulip"
    orange_wall_banner = "orange_wall_banner"
    orange_wool = "orange_wool"
    oxeye_daisy = "oxeye_daisy"
    oxidized_copper = "oxidized_copper"
    oxidized_cut_copper = "oxidized_cut_copper"
    oxidized_cut_copper_slab = "oxidized_cut_copper_slab"
    oxidized_cut_copper_stairs = "oxidized_cut_copper_stairs"
    packed_ice = "packed_ice"
    packed_mud = "packed_mud"
    pearlescent_froglight = "pearlescent_froglight"
    peony = "peony"
    petrified_oak_slab = "petrified_oak_slab"
    piglin_head = "piglin_head"
    piglin_wall_head = "piglin_wall_head"
    pink_banner = "pink_banner"
    pink_bed = "pink_bed"
    pink_candle = "pink_candle"
    pink_candle_cake = "pink_candle_cake"
    pink_carpet = "pink_carpet"
    pink_concrete = "pink_concrete"
    pink_concrete_powder = "pink_concrete_powder"
    pink_glazed_terracotta = "pink_glazed_terracotta"
    pink_petals = "pink_petals"
    pink_shulker_box = "pink_shulker_box"
    pink_stained_glass = "pink_stained_glass"
    pink_stained_glass_pane = "pink_stained_glass_pane"
    pink_terracotta = "pink_terracotta"
    pink_tulip = "pink_tulip"
    pink_wall_banner = "pink_wall_banner"
    pink_wool = "pink_wool"
    piston = "piston"
    piston_head = "piston_head"
    pitcher_crop = "pitcher_crop"
    pitcher_plant = "pitcher_plant"
    player_head = "player_head"
    player_wall_head = "player_wall_head"
    podzol = "podzol"
    pointed_dripstone = "pointed_dripstone"
    polished_andesite = "polished_andesite"
    polished_andesite_slab = "polished_andesite_slab"
    polished_andesite_stairs = "polished_andesite_stairs"
    polished_basalt = "polished_basalt"
    polished_blackstone = "polished_blackstone"
    polished_blackstone_brick_slab = "polished_blackstone_brick_slab"
    polished_blackstone_brick_stairs = "polished_blackstone_brick_stairs"
    polished_blackstone_brick_wall = "polished_blackstone_brick_wall"
    polished_blackstone_bricks = "polished_blackstone_bricks"
    polished_blackstone_button = "polished_blackstone_button"
    polished_blackstone_pressure_plate = "polished_blackstone_pressure_plate"
    polished_blackstone_slab = "polished_blackstone_slab"
    polished_blackstone_stairs = "polished_blackstone_stairs"
    polished_blackstone_wall = "polished_blackstone_wall"
    polished_deepslate = "polished_deepslate"
    polished_deepslate_slab = "polished_deepslate_slab"
    polished_deepslate_stairs = "polished_deepslate_stairs"
    polished_deepslate_wall = "polished_deepslate_wall"
    polished_diorite = "polished_diorite"
    polished_diorite_slab = "polished_diorite_slab"
    polished_diorite_stairs = "polished_diorite_stairs"
    polished_granite = "polished_granite"
    polished_granite_slab = "polished_granite_slab"
    polished_granite_stairs = "polished_granite_stairs"
    poppy = "poppy"
    potatoes = "potatoes"
    potted_acacia_sapling = "potted_acacia_sapling"
    potted_allium = "potted_allium"
    potted_azalea_bush = "potted_azalea_bush"
    potted_azure_bluet = "potted_azure_bluet"
    potted_bamboo = "potted_bamboo"
    potted_birch_sapling = "potted_birch_sapling"
    potted_blue_orchid = "potted_blue_orchid"
    potted_brown_mushroom = "potted_brown_mushroom"
    potted_cactus = "potted_cactus"
    potted_cherry_sapling = "potted_cherry_sapling"
    potted_cornflower = "potted_cornflower"
    potted_crimson_fungus = "potted_crimson_fungus"
    potted_crimson_roots = "potted_crimson_roots"
    potted_dandelion = "potted_dandelion"
    potted_dark_oak_sapling = "potted_dark_oak_sapling"
    potted_dead_bush = "potted_dead_bush"
    potted_fern = "potted_fern"
    potted_flowering_azalea_bush = "potted_flowering_azalea_bush"
    potted_jungle_sapling = "potted_jungle_sapling"
    potted_lily_of_the_valley = "potted_lily_of_the_valley"
    potted_mangrove_propagule = "potted_mangrove_propagule"
    potted_oak_sapling = "potted_oak_sapling"
    potted_orange_tulip = "potted_orange_tulip"
    potted_oxeye_daisy = "potted_oxeye_daisy"
    potted_pink_tulip = "potted_pink_tulip"
    potted_poppy = "potted_poppy"
    potted_red_mushroom = "potted_red_mushroom"
    potted_red_tulip = "potted_red_tulip"
    potted_spruce_sapling = "potted_spruce_sapling"
    potted_torchflower = "potted_torchflower"
    potted_warped_fungus = "potted_warped_fungus"
    potted_warped_roots = "potted_warped_roots"
    potted_white_tulip = "potted_white_tulip"
    potted_wither_rose = "potted_wither_rose"
    powder_snow = "powder_snow"
    powder_snow_cauldron = "powder_snow_cauldron"
    powered_rail = "powered_rail"
    prismarine = "prismarine"
    prismarine_brick_slab = "prismarine_brick_slab"
    prismarine_brick_stairs = "prismarine_brick_stairs"
    prismarine_bricks = "prismarine_bricks"
    prismarine_slab = "prismarine_slab"
    prismarine_stairs = "prismarine_stairs"
    prismarine_wall = "prismarine_wall"
    pumpkin = "pumpkin"
    pumpkin_stem = "pumpkin_stem"
    purple_banner = "purple_banner"
    purple_bed = "purple_bed"
    purple_candle = "purple_candle"
    purple_candle_cake = "purple_candle_cake"
    purple_carpet = "purple_carpet"
    purple_concrete = "purple_concrete"
    purple_concrete_powder = "purple_concrete_powder"
    purple_glazed_terracotta = "purple_glazed_terracotta"
    purple_shulker_box = "purple_shulker_box"
    purple_stained_glass = "purple_stained_glass"
    purple_stained_glass_pane = "purple_stained_glass_pane"
    purple_terracotta = "purple_terracotta"
    purple_wall_banner = "purple_wall_banner"
    purple_wool = "purple_wool"
    purpur_block = "purpur_block"
    purpur_pillar = "purpur_pillar"
    purpur_slab = "purpur_slab"
    purpur_stairs = "purpur_stairs"
    quartz_block = "quartz_block"
    quartz_bricks = "quartz_bricks"
    quartz_pillar = "quartz_pillar"
    quartz_slab = "quartz_slab"
    quartz_stairs = "quartz_stairs"
    rail = "rail"
    raw_copper_block = "raw_copper_block"
    raw_gold_block = "raw_gold_block"
    raw_iron_block = "raw_iron_block"
    red_banner = "red_banner"
    red_bed = "red_bed"
    red_candle = "red_candle"
    red_candle_cake = "red_candle_cake"
    red_carpet = "red_carpet"
    red_concrete = "red_concrete"
    red_concrete_powder = "red_concrete_powder"
    red_glazed_terracotta = "red_glazed_terracotta"
    red_mushroom = "red_mushroom"
    red_mushroom_block = "red_mushroom_block"
    red_nether_brick_slab = "red_nether_brick_slab"
    red_nether_brick_stairs = "red_nether_brick_stairs"
    red_nether_brick_wall = "red_nether_brick_wall"
    red_nether_bricks = "red_nether_bricks"
    red_sand = "red_sand"
    red_sandstone = "red_sandstone"
    red_sandstone_slab = "red_sandstone_slab"
    red_sandstone_stairs = "red_sandstone_stairs"
    red_sandstone_wall = "red_sandstone_wall"
    red_shulker_box = "red_shulker_box"
    red_stained_glass = "red_stained_glass"
    red_stained_glass_pane = "red_stained_glass_pane"
    red_terracotta = "red_terracotta"
    red_tulip = "red_tulip"
    red_wall_banner = "red_wall_banner"
    red_wool = "red_wool"
    redstone_block = "redstone_block"
    redstone_lamp = "redstone_lamp"
    redstone_ore = "redstone_ore"
    redstone_torch = "redstone_torch"
    redstone_wall_torch = "redstone_wall_torch"
    redstone_wire = "redstone_wire"
    reinforced_deepslate = "reinforced_deepslate"
    repeater = "repeater"
    repeating_command_block = "repeating_command_block"
    respawn_anchor = "respawn_anchor"
    rooted_dirt = "rooted_dirt"
    rose_bush = "rose_bush"
    sand = "sand"
    sandstone = "sandstone"
    sandstone_slab = "sandstone_slab"
    sandstone_stairs = "sandstone_stairs"
    sandstone_wall = "sandstone_wall"
    scaffolding = "scaffolding"
    sculk = "sculk"
    sculk_catalyst = "sculk_catalyst"
    sculk_sensor = "sculk_sensor"
    sculk_shrieker = "sculk_shrieker"
    sculk_vein = "sculk_vein"
    sea_lantern = "sea_lantern"
    sea_pickle = "sea_pickle"
    seagrass = "seagrass"
    shroomlight = "shroomlight"
    shulker_box = "shulker_box"
    skeleton_skull = "skeleton_skull"
    skeleton_wall_skull = "skeleton_wall_skull"
    slime_block = "slime_block"
    small_amethyst_bud = "small_amethyst_bud"
    small_dripleaf = "small_dripleaf"
    smithing_table = "smithing_table"
    smoker = "smoker"
    smooth_basalt = "smooth_basalt"
    smooth_quartz = "smooth_quartz"
    smooth_quartz_slab = "smooth_quartz_slab"
    smooth_quartz_stairs = "smooth_quartz_stairs"
    smooth_red_sandstone = "smooth_red_sandstone"
    smooth_red_sandstone_slab = "smooth_red_sandstone_slab"
    smooth_red_sandstone_stairs = "smooth_red_sandstone_stairs"
    smooth_sandstone = "smooth_sandstone"
    smooth_sandstone_slab = "smooth_sandstone_slab"
    smooth_sandstone_stairs = "smooth_sandstone_stairs"
    smooth_stone = "smooth_stone"
    smooth_stone_slab = "smooth_stone_slab"
    sniffer_egg = "sniffer_egg"
    snow = "snow"
    snow_block = "snow_block"
    soul_campfire = "soul_campfire"
    soul_fire = "soul_fire"
    soul_lantern = "soul_lantern"
    soul_sand = "soul_sand"
    soul_soil = "soul_soil"
    soul_torch = "soul_torch"
    soul_wall_torch = "soul_wall_torch"
    spawner = "spawner"
    sponge = "sponge"
    spore_blossom = "spore_blossom"
    spruce_button = "spruce_button"
    spruce_door = "spruce_door"
    spruce_fence = "spruce_fence"
    spruce_fence_gate = "spruce_fence_gate"
    spruce_hanging_sign = "spruce_hanging_sign"
    spruce_leaves = "spruce_leaves"
    spruce_log = "spruce_log"
    spruce_planks = "spruce_planks"
    spruce_pressure_plate = "spruce_pressure_plate"
    spruce_sapling = "spruce_sapling"
    spruce_sign = "spruce_sign"
    spruce_slab = "spruce_slab"
    spruce_stairs = "spruce_stairs"
    spruce_trapdoor = "spruce_trapdoor"
    spruce_wall_hanging_sign = "spruce_wall_hanging_sign"
    spruce_wall_sign = "spruce_wall_sign"
    spruce_wood = "spruce_wood"
    sticky_piston = "sticky_piston"
    stone = "stone"
    stone_brick_slab = "stone_brick_slab"
    stone_brick_stairs = "stone_brick_stairs"
    stone_brick_wall = "stone_brick_wall"
    stone_bricks = "stone_bricks"
    stone_button = "stone_button"
    stone_pressure_plate = "stone_pressure_plate"
    stone_slab = "stone_slab"
    stone_stairs = "stone_stairs"
    stonecutter = "stonecutter"
    stripped_acacia_log = "stripped_acacia_log"
    stripped_acacia_wood = "stripped_acacia_wood"
    stripped_bamboo_block = "stripped_bamboo_block"
    stripped_birch_log = "stripped_birch_log"
    stripped_birch_wood = "stripped_birch_wood"
    stripped_cherry_log = "stripped_cherry_log"
    stripped_cherry_wood = "stripped_cherry_wood"
    stripped_crimson_hyphae = "stripped_crimson_hyphae"
    stripped_crimson_stem = "stripped_crimson_stem"
    stripped_dark_oak_log = "stripped_dark_oak_log"
    stripped_dark_oak_wood = "stripped_dark_oak_wood"
    stripped_jungle_log = "stripped_jungle_log"
    stripped_jungle_wood = "stripped_jungle_wood"
    stripped_mangrove_log = "stripped_mangrove_log"
    stripped_mangrove_wood = "stripped_mangrove_wood"
    stripped_oak_log = "stripped_oak_log"
    stripped_oak_wood = "stripped_oak_wood"
    stripped_spruce_log = "stripped_spruce_log"
    stripped_spruce_wood = "stripped_spruce_wood"
    stripped_warped_hyphae = "stripped_warped_hyphae"
    stripped_warped_stem = "stripped_warped_stem"
    structure_block = "structure_block"
    structure_void = "structure_void"
    sugar_cane = "sugar_cane"
    sunflower = "sunflower"
    suspicious_gravel = "suspicious_gravel"
    suspicious_sand = "suspicious_sand"
    sweet_berry_bush = "sweet_berry_bush"
    tall_grass = "tall_grass"
    tall_seagrass = "tall_seagrass"
    target = "target"
    terracotta = "terracotta"
    tinted_glass = "tinted_glass"
    tnt = "tnt"
    torch = "torch"
    torchflower = "torchflower"
    torchflower_crop = "torchflower_crop"
    trapped_chest = "trapped_chest"
    tripwire = "tripwire"
    tripwire_hook = "tripwire_hook"
    tube_coral = "tube_coral"
    tube_coral_block = "tube_coral_block"
    tube_coral_fan = "tube_coral_fan"
    tube_coral_wall_fan = "tube_coral_wall_fan"
    tuff = "tuff"
    turtle_egg = "turtle_egg"
    twisting_vines = "twisting_vines"
    twisting_vines_plant = "twisting_vines_plant"
    verdant_froglight = "verdant_froglight"
    vine = "vine"
    void_air = "void_air"
    wall_torch = "wall_torch"
    warped_button = "warped_button"
    warped_door = "warped_door"
    warped_fence = "warped_fence"
    warped_fence_gate = "warped_fence_gate"
    warped_fungus = "warped_fungus"
    warped_hanging_sign = "warped_hanging_sign"
    warped_hyphae = "warped_hyphae"
    warped_nylium = "warped_nylium"
    warped_planks = "warped_planks"
    warped_pressure_plate = "warped_pressure_plate"
    warped_roots = "warped_roots"
    warped_sign = "warped_sign"
    warped_slab = "warped_slab"
    warped_stairs = "warped_stairs"
    warped_stem = "warped_stem"
    warped_trapdoor = "warped_trapdoor"
    warped_wall_hanging_sign = "warped_wall_hanging_sign"
    warped_wall_sign = "warped_wall_sign"
    warped_wart_block = "warped_wart_block"
    water = "water"
    water_cauldron = "water_cauldron"
    waxed_copper_block = "waxed_copper_block"
    waxed_cut_copper = "waxed_cut_copper"
    waxed_cut_copper_slab = "waxed_cut_copper_slab"
    waxed_cut_copper_stairs = "waxed_cut_copper_stairs"
    waxed_exposed_copper = "waxed_exposed_copper"
    waxed_exposed_cut_copper = "waxed_exposed_cut_copper"
    waxed_exposed_cut_copper_slab = "waxed_exposed_cut_copper_slab"
    waxed_exposed_cut_copper_stairs = "waxed_exposed_cut_copper_stairs"
    waxed_oxidized_copper = "waxed_oxidized_copper"
    waxed_oxidized_cut_copper = "waxed_oxidized_cut_copper"
    waxed_oxidized_cut_copper_slab = "waxed_oxidized_cut_copper_slab"
    waxed_oxidized_cut_copper_stairs = "waxed_oxidized_cut_copper_stairs"
    waxed_weathered_copper = "waxed_weathered_copper"
    waxed_weathered_cut_copper = "waxed_weathered_cut_copper"
    waxed_weathered_cut_copper_slab = "waxed_weathered_cut_copper_slab"
    waxed_weathered_cut_copper_stairs = "waxed_weathered_cut_copper_stairs"
    weathered_copper = "weathered_copper"
    weathered_cut_copper = "weathered_cut_copper"
    weathered_cut_copper_slab = "weathered_cut_copper_slab"
    weathered_cut_copper_stairs = "weathered_cut_copper_stairs"
    weeping_vines = "weeping_vines"
    weeping_vines_plant = "weeping_vines_plant"
    wet_sponge = "wet_sponge"
    wheat = "wheat"
    white_banner = "white_banner"
    white_bed = "white_bed"
    white_candle = "white_candle"
    white_candle_cake = "white_candle_cake"
    white_carpet = "white_carpet"
    white_concrete = "white_concrete"
    white_concrete_powder = "white_concrete_powder"
    white_glazed_terracotta = "white_glazed_terracotta"
    white_shulker_box = "white_shulker_box"
    white_stained_glass = "white_stained_glass"
    white_stained_glass_pane = "white_stained_glass_pane"
    white_terracotta = "white_terracotta"
    white_tulip = "white_tulip"
    white_wall_banner = "white_wall_banner"
    white_wool = "white_wool"
    wither_rose = "wither_rose"
    wither_skeleton_skull = "wither_skeleton_skull"
    wither_skeleton_wall_skull = "wither_skeleton_wall_skull"
    yellow_banner = "yellow_banner"
    yellow_bed = "yellow_bed"
    yellow_candle = "yellow_candle"
    yellow_candle_cake = "yellow_candle_cake"
    yellow_carpet = "yellow_carpet"
    yellow_concrete = "yellow_concrete"
    yellow_concrete_powder = "yellow_concrete_powder"
    yellow_glazed_terracotta = "yellow_glazed_terracotta"
    yellow_shulker_box = "yellow_shulker_box"
    yellow_stained_glass = "yellow_stained_glass"
    yellow_stained_glass_pane = "yellow_stained_glass_pane"
    yellow_terracotta = "yellow_terracotta"
    yellow_wall_banner = "yellow_wall_banner"
    yellow_wool = "yellow_wool"
    zombie_head = "zombie_head"
    zombie_wall_head = "zombie_wall_head"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scute', 'scute.internal']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'scutemc',
    'version': '0.0.7',
    'description': 'A Python library for making Minecraft datapacks.',
    'long_description': '![Title](/assets/title.png)\n\n[![Discord](https://img.shields.io/discord/1139873703556034620?label=discord)](https://discord.gg/RnRTjq24vX)\n[![GitHub Repo stars](https://img.shields.io/github/stars/t0rnato/scute?label=stars)](https://github.com/T0RNATO/scute)\n[![PyPI](https://img.shields.io/pypi/v/scute)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/scute)](https://pypi.org/project/scutemc/)\n\nA simple Python library for creating Minecraft datapacks with Python. (Very much in development)\n\n[Read the docs](https://t0rnato.github.io/scute),\n[view on PyPi](https://pypi.org/project/scutemc/), or\n[join the Discord](https://discord.gg/RnRTjq24vX) for support.',
    'author': 'TorNato',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/T0RNATO/scute',
    'packages': packages,
    'package_data': package_data,
}


setup(**setup_kwargs)



from uuid import uuid4

from scute.items import Item
from scute.recipes import ShapedCraftingLayout, registerRecipe, RecipeType
from scute import pack


def knowledgeBookCraft(outputItem: Item, recipe: ShapedCraftingLayout, dummyItem: str = "minecraft:knowledge_book"):
    """
    Creates a "crafting recipe" that results in an item with nbt, using the knowledge book trick.
    Args:
        outputItem: The output of the recipe
        recipe: The ingredients and layout of the recipe
        dummyItem: Optional. The item to use instead of the knowledge book.
    """
    rec = RecipeType.crafting_shaped(recipe, dummyItem)
    recipe_name = str(uuid4())
    registerRecipe(rec, pack.namespace, recipe_name)
    # Todo
